/**
Autore Renato Ricci <BR>
02-09-00 <BR>
Versione Beta <BR>
*/

package dati_ns;

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.String;

import utility_ns.*;
import main_ns.*;

/**
Classe contenente informazioni sul modo di esecuzione del Name Server, <BR>
se primario o secondario. <BR>
Fornisce le funzionalit per verificare se tale stato  coerente alla
presenza dell'altro Name Server e per allinearsi con esso.
*/
public class FlagPrimario
  {
	private Socket socket,socket_ns;
	
  private ObjectInputStream in=null,from_ns=null;
	
  private ObjectOutputStream out=null,to_ns=null;
	
  private MainClass mainclass=null;
	
	/** 
	Contiene lo stato di esecuzione del Name Server: 
	true se il Name Server  Primario, false se  Secondario
	*/
	private boolean primario;

  /**
  Costruttore: inizializza il Name Server a secondario. <BR>
	*/
  public FlagPrimario(MainClass mc)
	  {
		System.out.println("Name Server secondario in esecuzione...");
		this.primario=false;
		mainclass=mc;
		}
		
	/**
	Metodo: setta il Name Server a Primario. <BR>
	*/		
	public synchronized void setFlag()
    {
	  this.primario=true;
		System.out.println("Name Server diventa Primario"); 
	  }
		
	/**
	Metodo: restituisce lo stato di esecuzione del Name Server (true se primario). <BR>
	*/	
  public synchronized boolean getFlag()
    {
	  return this.primario;
	  }
			
	/**
	Metodo: verifica l'effettiva presenza dell'altro Name server. <BR>
	Ritorna lo stato del Name Server (true se primario). <BR>
	*/		
  public synchronized boolean verificaCaduta()
	  {
		if (!this.primario)
		  {
			System.out.println("Verifico la presenza del Name Server Primario");
		  tentaConnessione(); // Se sono Secondario verifico che il primario non sia caduto
			}
		return this.primario;
		}
		
	/**
	Metodo: tenta una connessione all'altro Name server. <BR>
	Se la connessione non riesce il Name Server viene settato a primario. <BR>
	*/		
	private void tentaConnessione()
		{
		try
		  {
			InetAddress indaltronameserver=InetAddress.getByName(mainclass.indnome);
			System.out.println("Tento una connessione con il Name Server Primario");
			Socket socket_ns=new Socket(indaltronameserver,mainclass.portaaltronameserver);
			System.out.println("Connessione riuscita, Name Server Primario presente");
			this.primario=false; // Se riesco a connettermi al Primario rimango Secondario
			System.out.println("Chiusa la connessione con il Name Server Primario");
			socket_ns.close();
		  }
		catch (IOException ioe1)
		  {
			System.out.println("Connessione non riuscita, divento Name Server Primario");
	    this.primario=true;  // Se non riesco a connettermi mi setto come nuovo Primario
			}
		catch (NullPointerException npe1)
		  {
			System.out.println("Connessione non riuscita, divento Name Server Primario");
      this.primario=true;  // Se non riesco a connettermi mi setto come nuovo Primario
			}
		/*catch (UnknownHostException uhe1)
		  {
			System.out.println("Connessione non riuscita, divento Name Server Primario");
	    this.primario=true;  // Se non riesco a connettermi mi setto come nuovo Primario
			}*/
		}

	/**
	Metodo: esegue recovery richiedendo la tabella dei Server di Chat 
	al Name Server primario. <BR>
	Restituisce un boolean: <BR>
	true se il recovery  riuscito, false altrimenti. <BR>
	*/		
  public boolean recovery()
    {
		System.out.println("NS2: Eseguo recovery...");
		int dim;
		boolean risultato=false;
		String nome_sdc=null;
		InetAddress ind_sdc=null;
		try
		  {
			InetAddress indaltronameserver=InetAddress.getByName(mainclass.indnome);
			socket_ns=new Socket(indaltronameserver,mainclass.portaaltronameserver);
			System.out.println("NS2: Connessione a NS1 eseguita...");
	    to_ns=new ObjectOutputStream(socket_ns.getOutputStream());
			System.out.println("NS2: Creato flusso di uscita...");
			from_ns=new ObjectInputStream(socket_ns.getInputStream());
			System.out.println("NS2: Creato flusso di ingresso...");
			TabellaServerDiChat copiatabsdc=new TabellaServerDiChat(mainclass);
			to_ns.writeObject("TABLEREQ");
			System.out.println("NS2: Richiesta tabella eseguita, attendo dati...");
			Integer dime=(Integer)from_ns.readObject();
			dim=dime.intValue();
			System.out.println("NS2: Ricevuta dimensione tabella="+dim);
			for(int i=0;i<dim;i++)
			  {
			  nome_sdc=(String)from_ns.readObject();
				ind_sdc=(InetAddress)from_ns.readObject();
				System.out.println("NS2: elemento "+i+": "+nome_sdc+" "+ind_sdc+" ricevuto");				
				if (copiatabsdc.addElement(nome_sdc,ind_sdc)<0)
				  System.out.println("NS2: Qualcosa non va, non riesco a riempire la tabella!");					
				}
			mainclass.tabsdc.setCopiaTabella(copiatabsdc);
			System.out.println("Tabella memorizzata");
			to_ns.close();
			System.out.println("NS2: chiusura flusso di uscita riuscito");
			from_ns.close();
			System.out.println("NS2: chiusura flusso di ingresso riuscito");
			socket_ns.close();
			System.out.println("NS2: chiusura socket riuscita");
		  risultato=true;
		  }
		catch (IOException ioe1)
		  {
			risultato=false;
      try
			  {
				System.out.println("Socket per recovery fallita");
			  to_ns.close();
			  from_ns.close();
				socket_ns.close();
				}
			catch (IOException ioe2)
			  {
				System.out.println("Chiusura socket per recovery fallita");
        }
			catch (NullPointerException npe1)
			  {
				System.out.println("Chiusura socket per recovery fallita, non c'!");
        }
			}
		catch (NullPointerException npe2)
		  {
			risultato=false;
			System.out.println("Socket per recovery fallita, non c'!");
			}
    /*catch (UnknownHostException uhe1)
		  {
			risultato=false;
			System.out.println("Socket per recovery fallita, Host sconosciuto");
		 	}*/
		catch (ClassNotFoundException cnfe1)
		  {
			}
		return risultato;
		}
		
  }  // fine classe

