/**
Autore Renato Ricci <BR>
29-08-00 <BR>
Versione Beta <BR>
*/

package dati_ns;

import java.lang.*;
import java.net.*;
import java.util.*;

import main_ns.*;

/**
Classe di gestione della Tabella dei Server di Chat. <BR>
*/
public class TabellaServerDiChat
  {
	/** Array contenente le informazioni sui Server di Chat registrati*/
	private InfoServerDiChat[] infosdc;

 	private MainClass mainclass=null;
		
	private static final int DIM_MAX=10;

	/**
	Costruttore della Tabella dei Server di Chat.  <BR>
	Inizializza tutto a null.<BR>
	*/	
	public TabellaServerDiChat(MainClass mc)
		{
		mainclass=mc;
		infosdc = new InfoServerDiChat[DIM_MAX];
		for (int i=0;i<DIM_MAX;i++)
		  infosdc[i]=null;
		}

  /** 
	Metodo: restituisce un valore che indica la dimensione della tabella 
	dei Server di Chat. <BR>
	*/
	public int size()
	  {
		int dimensione=0;
		int i=0;
		while ((i<DIM_MAX)&&(infosdc[i]!=null))
		  i++;
		dimensione=i;
		return dimensione;
		}
  
	/**
	Metodo: aggiunge un elemento alla tabella dei Server di Chat noti nome e indirizzo. <BR>
	Restitisce un valore che indica: <BR>
	 0 se l'operazione  andata a buon fine; <BR>
	-1 se c' stato errore perch la Tabella  piena; <BR>
	-2 se c' stato errore perch l'elemento  gi presente. <BR>
	Accede in modo esclusivo alla tabella. <BR>
	*/
  public synchronized int addElement(String nome_sdc,InetAddress indirizzo_sdc)
    {
		int dimensione=this.size();
		int i=0;
		if (dimensione==DIM_MAX)
		  return -1;                  // errore, tabella piena
		else
		  {
		  while ((i<dimensione)&&(infosdc[i].getName().compareToIgnoreCase(nome_sdc)!=0))
		    i++;
		  if (i==dimensione)          // Elemento non presente in tabella
			  {
				InfoServerDiChat elemento=new InfoServerDiChat(nome_sdc,indirizzo_sdc);
				addElementObject(elemento,i);
			  return 0;                 // operazione andata a buon fine
			  }
			else
			  return -2;                // errore, elemento gi presente
			}
		}

  /**
	Metodo: elimina dalla Tabella un Server di Chat noto il nome.<BR>
	Restitisce un valore:
   0 se l'operazione  andata a buon fine; <BR>
	-1 se c' stato errore perch l'elemento non  stato trovato. <BR>
	Accede in modo esclusivo alla tabella. <BR>
  */
	public synchronized int deleteElement(String nome_sdc)
	  {
		int i=0;
		int dimensione=this.size();
		while ((i<dimensione)&&(infosdc[i].getName().compareToIgnoreCase(nome_sdc)!=0))
		  i++;
		if (i==dimensione)
		  return -1;                 // errore, elemento non trovato
		else
		  {
			for (int j=i;j<DIM_MAX-1;j++)
			  infosdc[j]=infosdc[j+1]; // sposto tutti gli elementi avanti di una posizione
			infosdc[DIM_MAX-1]=null;   // l'ultimo elemento  sicuramente null
			return 0;                  // operazione andata a buon fine
			}
		}

  /**
	Metodo: restituisce il nome del Server di Chat nella posizione data. <BR>
	Accede in modo esclusivo alla tabella. <BR>
	*/
	public synchronized String getElementName(int posizione)
	  {
		int dimensione=this.size();
		if ((posizione<dimensione)&&(infosdc[posizione]!=null))
		  return infosdc[posizione].getName();
		else
		  return null;
		}
	
	/**
	Metodo: restituisce l'indirizzo del Server di Chat nella posizione data.
	Accede in modo esclusivo alla tabella. <BR>
	*/
  public synchronized InetAddress getElementAddress(int posizione)
	  {
		int dimensione=this.size();
		if ((posizione<dimensione)&&(infosdc[posizione]!=null))
	    return infosdc[posizione].getAddress();
	  else
		  return null;
		}

  /**
	Metodo: restituisce il nome del Server di Chat scegliendo la posizione in modo casuale. <BR>
	Accede in modo esclusivo alla tabella. <BR>
	*/
  public synchronized InetAddress getCasualElementAddress()
	  {
		int dimensione=this.size();
		if (dimensione>0)
		  {
			int posizione=mainclass.random.nextInt(dimensione);// genera un numero pseudocasuale tra 0 e dim-1
	    return infosdc[posizione].getAddress();
			}
	  else
		  return null;
		}
	
  /**
	Metodo: crea e restituisce una copia della Tabella dei Server di Chat. <BR>
	Accede in modo esclusivo alla tabella. <BR>
	*/
	public synchronized TabellaServerDiChat getCopiaTabella()
	  {
		TabellaServerDiChat buff=new TabellaServerDiChat(mainclass);
		int dimensione=size();
		for (int i=0;i<dimensione;i++)
		  {
		  buff.addElementObject(infosdc[i],i);
			System.out.println("Copia elemento in posizione "+i+" eseguita");	
			}
		return buff;
		}
		
  /**
	Metodo: prende una copia della Tabella dei Server di Chat e la memorizza. <BR>
	Accede in modo esclusivo alla tabella. <BR>
	*/
	public synchronized void setCopiaTabella(TabellaServerDiChat buff)
	  {
		int dimensione=buff.size();
		for (int i=0;i<dimensione;i++)
		  {
			infosdc[i]=buff.getElementObject(i);
			System.out.println("Elemento in posizione "+i+" memorizzato");
			}
		}
		
	/**
	Metodo: aggiunge un elemento alla tabella dei Server di Chat noto un oggetto
  contenente le informazioni del server di Chat. <BR>
	*/	
	private void addElementObject(InfoServerDiChat element,int pos)
	  {
		infosdc[pos]=element;
		}
		
	/**
	Metodo: elimina un elemento dalla tabella dei Server di Chat noto un oggetto
  contenente le informazioni del server di Chat. <BR>
	*/	
  private InfoServerDiChat getElementObject (int pos)
	  {
		return infosdc[pos];
		}
	
		
	} // fine classe
	


