/**
// Autore Renato Ricci <BR>
// 27-08-00 <BR>
// Versione Beta <BR>
*/

package utility_ns;

import java.io.*;
import java.net.*;
import java.util.*;
import dati_ns.*;
import main_ns.*;

/**
Classe che si occupa del riconoscimento della richiesta di un Client,
della elaborazione e della generazione della risposta. <BR>
*/
public class ClientReplier extends Thread
  {
  private Socket socket=null;
  private ObjectInputStream in=null;
  private ObjectOutputStream out=null;
  private MainClass mainclass;
	
	/**
	Costruttore: inizializza i flussi di ingresso e uscita verso il client. <BR>
	*/
  public ClientReplier(MainClass mc,Socket s)
	  {
		try
		  {
			mainclass=mc;
			socket=s;
			in=new ObjectInputStream(socket.getInputStream());
			System.out.println("Client Replier costr: creato flusso di ingresso...");
	    out=new ObjectOutputStream(socket.getOutputStream());   
			System.out.println("Client Replier costr: creato flusso di uscita...");
		  }
		catch (IOException ioe1)
		  {
			try
			  {
			  System.out.println("Client Replier costr: socket fallita");
				socket.close();
				}
			catch (IOException ioe2)
			  {
				System.out.println("Client Replier costr: chiusura socket fallita");
				}
			catch (NullPointerException npe1)
			  {
				System.out.println("Client Replier costr: socket inesistente, chiusura  fallita");
				}
			}
		catch (NullPointerException npe2)
		  {
			System.out.println("Client Replier costr: socket inesistente");
			}
		catch (ArrayIndexOutOfBoundsException aioobe)
		  {
			try
			  {
	  		System.out.println("Client Replier costr: accesso all'array errato");
				in.close();
				System.out.println("Client Replier costr: chiusura flusso di ingresso riuscita");
				out.close();
				System.out.println("Client Replier costr: chiusura flusso di uscita riuscita");
				socket.close();
				System.out.println("Client Replier costr: chiusura socket riuscita");
				}
			catch (IOException io3)
			  {
				System.out.println("Client Replier costr: chiusura socket non riuscita");
				}
			catch (NullPointerException npe3)
			  {
				System.out.println("Client Replier costr: socket inesistente, chiusura non riuscita");
				}
			}
		}
	
	/**
	Thread che riceve la richiesta di un Client, verifica se il Name Server
	 abilitato a rispondere, legge il tipo di richiesta e invoca il metodo
	che risponde a tale richiesta. <BR>
	*/
	public void run()
	  {
		try
		  {
			System.out.println("Client Replier: in esecuzione...");
			if (mainclass.primario.verificaCaduta())
			// Se sono effettivamente il Primario
			  { 
			  String str=(String)in.readObject();
			  System.out.println("Client Replier: ricevuta richiesta da C");
		    if (str.equals("SCREQC"))
				  {
					System.out.println("NS1: ricevuta richiesta indirizzo da C");
					generatoreIndirizzo();
					}
				else	        
	        System.out.println("Client Replier: richiesta da C non corretta");
				}
			else
	        System.out.println("Client Replier: richiesta da SDC a NS2 rifiutata");
			  	in.close();
			  	System.out.println("Client Replier: chiusura flusso di ingresso riuscita");
					out.close();
			  	System.out.println("Client Replier: chiusura flusso di uscita riuscita");
 					socket.close();
			  	System.out.println("Client Replier: chiusura socket riuscita");					
			}			
		catch (IOException ioe1)
		  {
			try
			  {
				System.out.println("Client Replier: socket fallita");
				in.close();
				System.out.println("Client Replier: chiusura flusso di ingresso riuscita");
				out.close();
				System.out.println("Client Replier: chiusura flusso di uscita riuscita");
				socket.close();
				System.out.println("Client Replier: chiusura socket riuscita");
				}
			catch (IOException ioe2)
			  {
				System.out.println("Client Replier: chiusura socket non riuscita");
				}
			catch (NullPointerException npe1)
			  {
				System.out.println("Client Replier: socket inesistente, chiusura non riuscita");
				}
			}
		catch (NullPointerException npe2)
		  {
			System.out.println("Client Replier: socket inesistente");
			}
		catch (ClassNotFoundException cnfe1)
		  {
			}
		catch (ArrayIndexOutOfBoundsException aioobe)
		  {
			try
			  {
	  		System.out.println("Client Replier: accesso all'array errato");
				in.close();
				System.out.println("Client Replier: chiusura flusso di ingresso riuscita");
				out.close();
				System.out.println("Client Replier: chiusura flusso di uscita riuscita");
				socket.close();
				System.out.println("Client Replier: chiusura socket riuscita");
				}
			catch (IOException io3)
			  {
				System.out.println("Client Replier: chiusura socket non riuscita");
				}
			catch (NullPointerException npe3)
			  {
				System.out.println("Client Replier: socket inesistente, chiusura non riuscita");
				}
			}
		}
		
	/**
	Metodo: genera un indirizzo di Server di Chat a cui il Client si collegher
	estraendolo dalla Tabella dei Server di Chat in modo casuale. <BR>
	*/
	private void generatoreIndirizzo()
	  {
		try
		  {
			System.out.println("NS1: Generatore indirizzo in esecuzione...");
			InetAddress ind=mainclass.tabsdc.getCasualElementAddress();
		  if (ind!=null)
			  {
			  System.out.println("NS1 (Gen Ind): indirizzo "+ind+" estratto dalla tabella");
		    out.writeObject(ind);
				System.out.println("NS1 (Gen Ind): indirizzo "+ind+" mandato");
				}
			else
			  {
				System.out.println("NS1 (Gen Ind): nessun Server di Chat registrato");
				// Il client non riceve nulla
				}
			in.close();
			System.out.println("NS1 (Gen Ind): chiusura flusso di ingresso riuscita");
			out.close();
			System.out.println("NS1 (Gen Ind): chiusura flusso di uscita riuscita");
			socket.close();
			System.out.println("NS1 (Gen Ind): chiusura socket riuscita");
			}
		catch (IOException ioe1)
		  {				
			try
			  {
				System.out.println("NS1 (Gen Ind): socket fallita");
				in.close();
				System.out.println("NS1 (Gen Ind): chiusura flusso di ingresso riuscita");
				out.close();
				System.out.println("NS1 (Gen Ind): chiusura flusso di uscita riuscita");
				socket.close();
				System.out.println("NS1 (Gen Ind): chiusura socket riuscita");
				}
			catch (IOException ioe2)
			  {
				System.out.println("NS1 (Gen Ind): chiusura socket fallita");
				}
			catch (NullPointerException npe1)
			  {
				System.out.println("NS1 (Gen Ind): socket inesistente, chiusura fallita");
				}
			}
		catch (NullPointerException npe2) // vale anche per l'InetAddress?
		  {
			System.out.println("NS1 (Gen Ind): socket inesistente"+npe2);
			}
		catch (ArrayIndexOutOfBoundsException aioobe)
		  {
			try
			  {
	  		System.out.println("NS1 (Gen Ind): accesso all'array errato");
				in.close();
				System.out.println("NS1 (Gen Ind): chiusura flusso di ingresso riuscita");
				out.close();
				System.out.println("NS1 (Gen Ind): chiusura flusso di uscita riuscita");
				socket.close();
				System.out.println("NS1 (Gen Ind): chiusura socket riuscita");
				}
			catch (IOException io3)
			  {
				System.out.println("NS1 (Gen Ind): chiusura socket non riuscita");
				}
			catch (NullPointerException npe3)
			  {
				System.out.println("NS1 (Gen Ind): socket inesistente, chiusura non riuscita");
				}
			}
	  }

  }  // fine classe
