package scthreads;

import java.util.*;
import java.net.*;
import java.io.*;

import mainpack.*;
import strutturedati.*;
import utilita.*;

/**
Autore:	Terenzio<BR>
Data:		01-09-2000<BR>
Vers:		1.0<BR>
Classe che permette di risolvere le richieste dei Client.
*/

public class ClientReplier extends Thread {
	private MainClass mc;
	private DBStanzeLocali dbs;
	private CacheStanze cs;
	private Socket client; //socket a cui  connesso il Client
	private ObjectInputStream ois; //da qui leggo le informaziomi inviate dal Client
	private ObjectOutputStream oos;//qui invio le informazioni verso il Client
	//Struttura dati necessarie alla comunicazione con il name server
	private InfoServer[] serverchatremoti=null;
	
	//Costruttore
	
	/**
	Apre i flussi di I/O verso il Client.
	*/
	public ClientReplier(MainClass mc, Socket client) {
		this.mc=mc;
		this.client=client;
		dbs=mc.dbstanzelocali;
		cs=mc.cachestanze;
		try {
			ois=new ObjectInputStream(client.getInputStream());
			oos=new ObjectOutputStream(client.getOutputStream());
			System.out.println("CR - (Costr) Aperta connessione con C.");
		}
		catch (IOException ioe1) {
			//per debugging
			System.out.println("CR - (Costr) Impossibile aprire gli stream di I/O.");
			try {
				ois.close();
				oos.close();
				client.close();
			}
			catch (IOException ioe2) {
				//per debugging
				System.out.println("CR - (Costr) Impossibile chiudere la socket verso C.");
			}
			ois=null;
			oos=null;
			client=null;
		}
	} //fine Costruttore
	
	/**
	Verifica il tipo di richiesta effettuata da client e la risolve.
	*/
	public void run() {
		System.out.println("CR - (run) in esecuzione.");
		String info="NON INIZIALIZZATA A NULLA";
		boolean ok=true;
		if (ois!=null && oos!=null && client!=null) { //ha senso procedere
			try {
				info=(String)ois.readObject();
				System.out.println("CR - (run) Identificata richiesta da parte del C: "+info);
			}
			catch (Exception e) {
				//per debugging
				System.out.println("CR - (run) Impossibile leggere la richiesta del C.");
				try {
					ok=false;  //non ha pi senso procedere
					ois.close();
					oos.close();
					client.close();
				}
				catch (IOException ioe) {
					//per debugging
					System.out.println("CR - (run) Impossibile chiudere la comunicazione con C dopo il fallimento della lettura della richiesta.");
				}
			}
			if (ok) {
				if (info.equals(CostantiCom.ROOMREQC)) richiestaLista();
				if (info.equals(CostantiCom.ROOMACCC)) accessoCreazioneStanza();
				//terminate le operazioni del caso chiudo gli stream e la socket.
				try {
					ois.close();
					oos.close();
					client.close();
				}
				catch (IOException ioe) {
					//per debugging
					System.out.println("CR - (run) Impossibile chiudere la comunicazione con C dopo aver eseguito la richiesta.");
				}
			}
		} //fine if
	} //fine run
	
	/** 
	Metodo ptivato che permette di eseguire la richiesta della lista delle stanze nel sistema da parte 
	del	client. Se la cache  scaduta e non  ancora sotto aggiornamento la si aggiorna. Se questo metodo
	viene invocato certamente la socket e gli stream di I/O verso il client risultano aperti
	*/
	private void richiestaLista() {
		CacheStanze agg=null;
		String[] nomistanze=null;
		if (cs.verifyAggiornamento(CostantiAlt.DELAY)) { //bisogna aggiornare la cache
			System.out.println("CR - (List Req.) CacheStanze scaduta, tentativo di aggiornamento...");
			agg=aggiornamentoCache();
			if (agg==null) System.out.println("CR - (List Req.) Aggiornamento CacheStanze fallito.");
			else {
				cs.setAggiornamento(agg);
				System.out.println("CR - (List. Req.) Aggiornamento CacheStanze avventuo.");
			}
		}
		nomistanze=cs.getNomiStanze();
		//invio della lista
		if (nomistanze!=null) 
			try {
				oos.writeObject(new Integer(nomistanze.length));
				System.out.println("CR - (List Req.) Inviata lunghezza della lista a C, lunghezza: "+nomistanze.length); 
				for (int i=0; i<nomistanze.length; i++) {
					oos.writeObject(nomistanze[i]);
					System.out.println("CR - (List Req.) Inviata stanza numero "+i+" di nome: "+nomistanze[i]);
				}
			}
			catch (IOException ioe) {
				//per debugging
				System.out.println("CR - (List Req.) Impossibile inviare a C la lista stanze nel sistema.");
			}
		else 
			try {
				oos.writeObject(new Integer(CostantiCom.LISTAVUOTA));
				System.out.println("CR - (List Req.) Inviata LISTAVUOTA a C");
			}
			catch (IOException ioe) {
				//per debugging
				System.out.println("CR - (List Req.) Impossibile inviare il messaggio di LISTAVUOTA.");
			}
	} //fine richiestaLista
	
	/**
	Metodo che esegue l'aggiornamento della CacheStanze. Ritorna la nuova cahce se l'aggiornamento  
	riusciro null	altrimenti.
	*/
	private CacheStanze aggiornamentoCache() {
		//La nuova cache
		CacheStanze newcache=new CacheStanze();
		//comunicazione con il name server per ottenere gli indirizzi degli altri server di chat
		serverchatremoti=getListaServerChat();
		for (int i=0;i<serverchatremoti.length; i++) {
			//Mi faccio dare da ogni Server di Chat la lista delle stanze locali
			System.out.println("CR - (A. Cache) Richiesta la lista delle stanze a: "+serverchatremoti[i].getNome());
			String listastanze[]=getListaStanzeLocali(serverchatremoti[i]);
			if (listastanze!=null) {
				System.out.println("CR - (A. Cache) Lista stanze in "+serverchatremoti[i].getNome()+" non vuota.");
				InetAddress indirizzo=serverchatremoti[i].getIndirizzo();
				for (int j=0; j<listastanze.length; j++) {
					System.out.println("CR - (A. Cache) Aggiungo la stanza "+listastanze[j]+" alla cache che sto creando.");
					newcache.addStanza(listastanze[j],indirizzo);
				}
			}
		}
		return newcache;
	} //fine aggiornmanentoCache
	
	/**
	Comunica con il NameServer per ottenere la lista di tutti i server di chat presenti nel sistema.
	Ritorna null se qualcosa va storto.
	*/
	private InfoServer[] getListaServerChat() {
		Socket ns=null;
		ObjectInputStream inns=null;
		ObjectOutputStream outns=null;
		InfoServer[] listaserver=null;
		int lunghezzalista=-1;
		String nomeserverchat=null;
		InetAddress indirizzoserverchat=null;
				
		for (int i=0; i<2; i++) {
			//Connessione a NameServer
			NameServer nameserver=mc.getNameServer();
			String indns=nameserver.getIndirizzo();
			int portans=nameserver.getPorta();
			try {
				ns=new Socket(indns,portans);
				outns=new ObjectOutputStream(ns.getOutputStream());
				inns=new ObjectInputStream(ns.getInputStream());
				System.out.println("CR - (SC List Req.) Apertura connessione verso NS "+indns+" : "+portans+" avvenuta correttamente.");
			}
			catch (IOException ioe) {
			//per debugging
			System.out.println("CR - (SC List Req.) Impossibile aprire connessione verso NS "+indns+" : "+portans);
			System.out.println("CR - (SC List Req.) Il NSP di default  caduto, provo sul NSS."); 
			mc.switchNS();
			continue;
			}
			//Comunicazione con il NameServer per ottenere la Lista dei server di chat
			try {
				outns.writeObject(CostantiCom.SCREQS);
				System.out.println("CR - (SC List Req.) Inviato SCREQS a NS");
				lunghezzalista=((Integer)inns.readObject()).intValue();
				System.out.println("CR - (SC List Req.) Letta lunghezza della lista che sar inviata da NS, lunghezza: "+lunghezzalista);
				if (lunghezzalista<=0) {
					System.out.println("CR - (SC List Req.) Impossibile leggere la lunghezza della lista.");
					break;
				}
				listaserver=new InfoServer[lunghezzalista];
				for (int j=0; j<lunghezzalista; j++) {
					nomeserverchat=(String)inns.readObject();
					indirizzoserverchat=(InetAddress)inns.readObject();
					System.out.println("CR - (SC List. Req.) Letto SC numero "+j+" - nome: "+nomeserverchat+" indirizzo: "+indirizzoserverchat);
					listaserver[j]=new InfoServer(nomeserverchat,indirizzoserverchat);
					
				}
				break;
			}
			catch (Exception e) {
				//per debugging
				System.out.println("CR - (SC List Req.) Impossibile ricevere lista Server di Chat da NS.");
				System.out.println("CR - (SC List Req.) Eccezione ricevuta: "+e);
				try {
					outns.close();
					inns.close();
					ns.close();
					System.out.println("CR - (SC List Req.) Chiusura verso NS corretta anche se non si  ricevuta la lista dei SC.");
				}
				catch (IOException ioe) {
					//per debugging 
					System.out.println("CR - (SC List Req.) Problema chiusura comunicazione verso NS a seguito del fallimento della ricezione della lista dei SC.");
					//System.exit(-4);
				}
				mc.switchNS();
				listaserver=null;
				break;
			}
		} //fine for
		//Chiusura
		try {
			outns.close();
			inns.close();
			ns.close();
			System.out.println("CR - (SC List Req.) Chiusura corretta comunicazione verso NS.");

		}
		catch (IOException ioe) {
			//per debugging 
			System.out.println("CR -(SC List Req) Problema chiusura comunicazione con NS. ");
			//System.exit(-4);
		}
		return listaserver;
	} //fine getListaServerChat
	
	/**
	Metodo che date le informazioni relative ad un Server di Chat permette di avere la lista dei nomi
 	delle	stanze locali a quel Server. Ritorna null in caso di problemi.
	*/
	private String[] getListaStanzeLocali(InfoServer infoserver) {
		Socket scremoto=null;
		ObjectInputStream inscremoto=null;
		ObjectOutputStream outscremoto=null;
		String[] listatemp=null;
		if ( infoserver.getNome().equals(mc.nomeserver) ) {
				System.out.println("CR - (List Req.) Identificato server locale, ricerca lista...");
				//siamo sul server locale
				return dbs.getNomiStanze();
		}
		//dobbiamo comunicare con un server remoto
		//Apertura connessione con il ServerChat remoto
		try {
			scremoto=new Socket(infoserver.getIndirizzo(),CostantiAlt.SCFORSC);
			outscremoto=new ObjectOutputStream(scremoto.getOutputStream());			
			inscremoto=new ObjectInputStream(scremoto.getInputStream());
			System.out.println("CR - (List Req.) Aperta comunicazione verso il SC remoto "+infoserver.getNome()+" ind: "+infoserver.getIndirizzo());
		}
		catch (IOException ioeopen) {
			//per debugging
			System.out.println("CR - (List Req.) Impossibile aprire la comunicazione con il SC remoto "+infoserver.getNome()+" ind: "+infoserver.getIndirizzo());
			return null;
		}
		// Qui la socket e gli stream non saranno mai null
		// Comunicazione con il ServerChat remoto
		try {
			outscremoto.writeObject(CostantiCom.ROOMREQS);
			System.out.println("CR - (List Req.) Inviata ROOMREQS verso "+infoserver.getNome()+" ind: "+infoserver.getIndirizzo());
			int n=((Integer)inscremoto.readObject()).intValue();
			// potrei controllare se n vale CostantiCom.LISTAVUOTA
			System.out.println("CR - (List Req.) Ricevuta lunghezza lista stanze locali, lunghezza: "+n);
			if (n>0) listatemp=new String[n];
			for (int i=0; i<n; i++) {
				listatemp[i]=(String)inscremoto.readObject();
				System.out.println("CR - (List Req.) Ricevuta stanza numero "+i+" di nome: "+listatemp[i]);
			}
		}
		catch (Exception e) {
			//per debugging
			System.out.println("CR - (List Req.) Errore di comunicazione con il SC remoto.");
		}
		// Chiusura comunicazione con il ServerChat remoto
		try {
			inscremoto.close();
			outscremoto.close();
			scremoto.close();
			System.out.println("CR - (List Req.) Chiusura comunicazione con SC remoto avvenuta correttamente."); 
		}
		catch (IOException ioeclose) {
			//per debugging
			System.out.println("CR - (List Req.) Errore chiusura della comunicazione con il SC remoto.");
		}
		return listatemp;
	} //fine getListaStanzeLocali
	
	
	/** 
	Metodo ptivato che permette di eseguire la richiesta di accesso e/o creazione di una stanza nel
  sistema da parte del Client.
	*/
	private boolean accessoCreazioneStanza() {
		//variabili ausiliarie
		InetAddress host1=null,host2=null;
		
		// variabili per raccogliere i dati inviati dal Client
		String nomestanza=null,nomeutente=null;
		int portaudp=-1;
		InetAddress indirizzoutente=null;
		
		// recupero i dati inviati dal Client
		try {
			nomestanza=(String)ois.readObject();
			System.out.println("CR - (AC Stanza) Ricevuto da C nome stanza: "+nomestanza);
			nomeutente=(String)ois.readObject();
			System.out.println("CR - (AC Stanza) Ricevuto da C nome utente: "+nomeutente);
			portaudp=((Integer)ois.readObject()).intValue();
			System.out.println("CR - (AC Stanza) Ricevuto da C porta udp utente: "+portaudp);
			indirizzoutente=client.getInetAddress();
			System.out.println("CR - (AC Stanza) L'utente ha indirizzo: "+indirizzoutente);
		}
		catch (Exception e) {
			//per debugging
			System.out.println("CR - (AC Stanza) Impossibile acquisire dal C dati necessari per acc-cre stanza.");
			return false;
		}
		
		//Mi faccio dare la lista aggiornata delle stanze nel sistema e verifico la presenza della stanza
		//a cui l'utente vuole accedere.
		CacheStanze newcache=aggiornamentoCache();
		if (newcache==null) {
			System.out.println("CR - (AC Stanza) Impossibile verificare la presenza della stanza "+nomestanza+" nel sistema.");
			return false; //non ha senso proseguire
		}
		InfoGlobaleStanza igs=newcache.getStanza(nomestanza);
			if (igs!=null) { //la stanza esiste gi
				System.out.println("CR - (AC Stanza) La stanza "+nomestanza+" esiste gi nel sistema.");
				host1=igs.getServer1();
				host2=igs.getServer2();
			}
			cs.setAggiornamento(newcache); //gi che ce l'ho aggiorno la cache
			System.out.println("CR - (AC Stanza) Aggiornamento CacheStanze avventuo.");
			return accStanza(nomestanza,nomeutente,indirizzoutente,portaudp,host1,host2);
		} //fine accessoCreazioneStanza
			
	/**
	Metodo che realizza l'accesso di un utente ad una stanza. Eventualmente se la stanza nonesiste la 
	crea e se identifica una situazione di recovery predispone il tutto in modo tale che il recovery
	avvenga. Ritorna true se l'operazione va a buon fine, false in caso di problemi.
	*/
	private boolean accStanza(String nomestanza, String nomeutente, InetAddress indirizzoutente,int portaudp, InetAddress host1, InetAddress host2) {
		InfoCopia copia1=null,copia2=null;
		InetAddress local=null;
		if (host1!=null && host2!=null) { //la stanza esiste in duplice copia
			if (mc.rgen.nextBoolean()) {
				InetAddress temp=host1;
				host1=host2;
				host2=temp;
			}
			copia1=accesso(nomestanza,nomeutente,indirizzoutente,portaudp,true,host1);
			copia2=accesso(nomestanza,nomeutente,indirizzoutente,portaudp,false,host2);
		}
		if (host1!=null && host2==null) { //la stanza esiste in una sola copia su host1
			copia1=copia2=accesso(nomestanza,nomeutente,indirizzoutente,portaudp,false,host1);
		}
		if (host1==null && host2!=null) { //la stanza esiste in una sola copia su host2
			copia1=copia2=accesso(nomestanza,nomeutente,indirizzoutente,portaudp,false,host2);
		}
		if (host1==null && host2==null) { //la stanza non esiste e va creata
			//poich collegato==true questa  la creazione locale
			System.out.println("CR - (Acc. Stanza) La stanza "+nomestanza+" non  stata trovata provo a crearla..");
			copia1=creLocale(nomestanza,nomeutente,indirizzoutente,portaudp); 
			//questa  la creazione remota
			if (copia1!=null) { 
				System.out.println("CR - (Acc. Stanza) Stanza "+nomestanza+" creata sul server locale.");
				System.out.println("CR - (Acc. Stanza) vi  stato aggiunto l'utente "+nomeutente+" in ascolto su: "+indirizzoutente+" : "+portaudp);
				copia2=creRemota(nomestanza,copia1.getPorta(),nomeutente,indirizzoutente,portaudp);
				// una volta che si hanno le informazioni relative alla copia remota bisogna settarle nel 
				// Database delle Stanze Locali
				dbs.setInfoCopiaStanza(nomestanza,copia2);
			}
		}
		
		if (copia1==null || copia2==null) { //la creazione-accesso non  andata a buon fine
			try {
				System.out.println("CR - (Acc. Stanza) Stanza non creata o l'utente non vi  stato aggiunto! Probabile errore nei DB!");
				oos.writeObject(CostantiCom.FAIL);
			}
			catch (IOException e) {
				//per debugging
				System.out.println("CR - (Acc. Stanza) Impossibile comunicare il fallimento di acc-cre stanza.");
			}
			return false;
		}
		//l' accesso e/o creazione sono andati a buon fine
		try {
				oos.writeObject(CostantiCom.OK);
				System.out.println("CR - (Acc. Stanza) Inviato OK a C per operazione di acc-cre Stanza");
				oos.writeObject(copia1.getIndirizzo());
				System.out.println("CR - (Acc. Stanza) Inviato a C indirizzo stanza: "+copia1.getIndirizzo());
				oos.writeObject(new Integer(copia1.getPorta()));
				System.out.println("CR - (Acc. Stanza) Inviato a C porta stanza: "+copia1.getPorta());
				oos.writeObject(copia2.getIndirizzo());
				System.out.println("CR - (Acc. Stanza) Inviato a C indirizzo copia stanza: "+copia2.getIndirizzo());
				oos.writeObject(new Integer(copia2.getPorta()));
				System.out.println("CR - (Acc. Stanza) Inviato a C porta copia stanza: "+copia2.getPorta());
				return true;
			}
			catch (IOException e) {
				//per debugging
				System.out.println("CR - (Acc. Stanza) Impossibile inviare a C le informazioni relative alla stanza e alla sua copia.");
				return false;
			}
	} //fine accStanza
			
	/**
	Implementa i controlli necessari all' accesso di una stanza gi esistente. La stanza pu essere
	sia remota che locale.Ritorna le informazioni relative alla copia acceduta se l'accesso va a buon
 	fine,	null altrimenti.
	*/
	private InfoCopia accesso(String nomestanza, String nomeutente, InetAddress indirizzoutente, int portaudp, boolean collegato, InetAddress hostdiaccesso) {
		InetAddress local=null;
		try {
			local=InetAddress.getLocalHost();
		}
		catch (UnknownHostException uoe) {
			//per debugging
			System.out.println("CR - (Acc.) Impossibile risolvere l'indirizzo del localhost");
			return null;
		}
		if (hostdiaccesso.equals(local)) {
			System.out.println("CR - (Acc.) L' accesso a "+nomestanza+"  locale");
			int portalocale=-1;
			portalocale=accLocale(nomestanza,nomeutente,indirizzoutente,portaudp,collegato);	//l'accesso  ad una stanza locale
			if (portalocale==-1) return null; //l'accesso locale non  andato a buon fine
			return new InfoCopia(hostdiaccesso,portalocale);
		}
		// qui l'accesso  ad una stanza remota
		return accRemoto(nomestanza,nomeutente,indirizzoutente,portaudp,collegato,hostdiaccesso);
	}  //fine accesso
	
	/**
	Implementa i controlli necessari alla creazione e all' accesso sul server locale di una stanza non 
	ancora esistente. Ritorna le informazioni relative alla copia creata e acceduta se l'operazione va
	a buon fine, null altrimenti.
	*/	
	private InfoCopia creLocale(String nomestanza, String nomeutente, InetAddress indirizzoutente, int portaudp) {
		System.out.println("CR - (Cre Loc) Creazione della stanza "+nomestanza+" iniziata."); 
		InetAddress local=null;
		InfoUtente utente=new InfoUtente(nomeutente, indirizzoutente, portaudp, true);
		GestoreStanza gs=new GestoreStanza(mc,nomestanza);
		int portastanza=gs.getPorta();
		InfoStanza infostanza=new InfoStanza(nomestanza,portastanza);
		try {
			local=InetAddress.getLocalHost();
		}
		catch (UnknownHostException uoe) {
			//per debugging
			System.out.println("CR - (Cre Loc) Impossibile risolvere l'indirizzo del localhost.");
			return null;
		}
		if (dbs.addStanza(infostanza)) 
			if (dbs.addUtenteToStanza(nomestanza,utente)) {
				gs.start();
				System.out.println("CR - (Cre Loc) Stanza "+nomestanza+" creata localemente.");
				return new InfoCopia(local,dbs.getPortaStanza(nomestanza));
			}
		//se arrivo qui allora l'operazione di creazione di una stanza e aggiunta di un utente  fallita
		System.out.println("CR - (Cre Loc) Problemi nella creazione locale della stanza "+nomestanza);
		return null;
	}  //fine creLocale
	
	/**
	Implementa i controlli necessari alla creazione e all' accesso su un server remoto di una stanza non 
	ancora esistente. Ritorna le informazioni relative alla copia creata e acceduta se l'operazione va
	a buon fine, null altrimenti.
	*/	
	private InfoCopia creRemota(String nomestanza, int portalocale,String nomeutente, InetAddress indirizzoutente, int portaudp) {
		// variabili necessarie per comunicare con l'host remoto	
		Socket sc;
		ObjectInputStream insc;
		ObjectOutputStream outsc;
		String messaggio=null;
		int portaremota=-1;
		InetAddress local=null,host=null;
		
		//variabili per trovare l'host remoto
		InfoServer serverremoto=null;
		boolean ricerca=true;
		
		//parametro di ritorno
		InfoCopia infocopia=null;
				
		try {
			local=InetAddress.getLocalHost();
		}
		catch (UnknownHostException uoe) {
			//per debugging
			System.out.println("CR - (Cre Rem) Impossibile risolvere l'indirizzo del localhost");
			return null;
		}
		if (serverchatremoti==null) return null;  //errore nella lista dei server di chat
		while (ricerca) { //inizio while
			serverremoto=serverchatremoti[mc.rgen.nextInt(serverchatremoti.length)];
			host=serverremoto.getIndirizzo();
			if (host.equals(local)) {
				System.out.println("CR - (Cre Rem) Scelto come server remoto il server locale, ritento...");
				continue;
			}
			System.out.println("CR - Server su cui sar tentata la creazione stanza:"+serverremoto.getNome()+" "+serverremoto.getIndirizzo());

			//Connessione al Server di Chat.
			try {
				sc=new Socket(host, CostantiAlt.SCFORSC);
				outsc=new ObjectOutputStream(sc.getOutputStream());
				insc=new ObjectInputStream(sc.getInputStream());
				System.out.println("CR - (Cre Rem) Connessione stabilita con: "+serverremoto.getNome()+" a "+serverremoto.getIndirizzo());
			}
			catch (IOException ioe) {
				//per debugging
				System.out.println("CR - (Cre Rem) Impossibile connettersi a: "+serverremoto.getNome()+" a "+serverremoto.getIndirizzo()+" cerco un altro SC...");
				continue; //ricerco un altro server.
			}
			//Comunicazione con SC
			try {
				outsc.writeObject(CostantiCom.ROOMCRS); //richiesta creazione
				System.out.println("CR - (Cre Rem) Inviato ROOMCRS verso "+serverremoto.getNome());
				outsc.writeObject(nomestanza); // nome stanza a cui si vuole creare
				System.out.println("CR - (Cre Rem) Inviato "+nomestanza+" verso "+serverremoto.getNome());
				outsc.writeObject(new Integer(portalocale)); //porta locale della stanza
				System.out.println("CR - (Cre Rem) Inviato "+portalocale+" verso "+serverremoto.getNome());
				outsc.writeObject(nomeutente); //nome dell'utente che vi vuole accedere
				System.out.println("CR - (Cre Rem) Inviato "+nomeutente+" verso "+serverremoto.getNome());
				outsc.writeObject(indirizzoutente); //indirizzo dell' utente
				System.out.println("CR - (Cre Rem) Inviato "+indirizzoutente+" verso "+serverremoto.getNome());
				outsc.writeObject(new Integer(portaudp)); //porta udp
				System.out.println("CR - (Cre Rem) Inviato "+portaudp+" verso "+serverremoto.getNome());
				
				outsc.writeObject(new Boolean(false)); //informazione se l'utente  collegato o meno
				System.out.println("CR - (Cre Rem) Inviato false verso "+serverremoto.getNome());
				messaggio=(String)insc.readObject();
				System.out.println("CR - (Cre Rem) Letto "+messaggio+" da "+serverremoto.getNome());
				if (messaggio!=null && (messaggio.compareTo(CostantiCom.OK)==0)) {
					portaremota=((Integer)insc.readObject()).intValue();
					System.out.println("CR - (Cre Rem) Letto "+portaremota+" da "+serverremoto.getNome());
					if (portaremota==-1) return null;  //la porta  errata
					infocopia=new InfoCopia(host,portaremota);
					ricerca=false;
					continue;
					}
				if (messaggio!=null && (messaggio.compareTo(CostantiCom.FAIL)==0)){
					System.out.println("CR - (Cre Rem) Problemi su SC remoto durante la creazione della stanza "+nomestanza+" cerco un altro server...");
					//cerco un' altro server
					continue;
				}
			}
			catch (Exception e) {
				//per debugging 
				System.out.println("CR - (Cre Rem) Comunicazione con SC fallita.");
				return null;
			}
			//Chiusura del Server di Chat.
			try {
				outsc.close();
				insc.close();
				sc.close();
				System.out.println("CR - Cre Rem) Chiusura comunicazione con SC avvenuta correttamente.");
			}
			catch (IOException ioe) {
				//per debugging 
				System.out.println("CR - (Cre Rem) Chiusura comunicazione SC remoto fallita.");
				return null;
		}
	} //fine while 
	return infocopia;
	} //fine creRemota
	
	/**
	Inserisce effettivamente l'utente che vuole accedere ad una stanza nel DBStanzeLocali. Ritorna la 
	porta della stanza se l'operazione va a buon fine -1 altrimenti.
	*/
	private int accLocale(String nomestanza,String nome, InetAddress indirizzo, int portaudp, boolean collegato) {
		InfoUtente utente=new InfoUtente(nome,indirizzo, portaudp, collegato);
		if (dbs.addUtenteToStanza(nomestanza, utente)) {
			System.out.println("CR - (Acc. Loc) L'utente "+nome+"  stato inserito in "+nomestanza);
			return dbs.getPortaStanza(nomestanza);
		}	
		return -1;
	} //fine accLocale
	
	/**
	Inserisce effettivamente l'utente che vuole accedere ad una stanza nel Server di Chat remoto passato
	attraverso il parametro host. Ritorna la porta della stanza remota se l'operazione va a buon fine, 
	null in caso di problemi.
	*/	
	private InfoCopia accRemoto(String nomestanza,String nome, InetAddress indirizzo, int portaudp, boolean collegato, InetAddress host) {
		Socket sc;
		ObjectInputStream insc;
		ObjectOutputStream outsc;
		String messaggio=null;
		int porta=-1;
		InfoCopia infocopia=null;
		
		//Connessione al Server di Chat.
		try {
			sc=new Socket(host, CostantiAlt.SCFORSC);
			outsc=new ObjectOutputStream(sc.getOutputStream());
			insc=new ObjectInputStream(sc.getInputStream());
		}
		catch (IOException ioe) {
			//per debugging
			System.out.println("CR - (Acc. Rem) Impossibile connettersi al SC per richiede l'accesso ad una stanza.");
			return null;
		}
		//Comunicazione con SC
		try {
			outsc.writeObject(CostantiCom.ROOMACCS); //richiesta accesso
			System.out.println("CR - (Acc. Rem) inviato a SC ROOMACCS.");
			outsc.writeObject(nomestanza); // nome stanza a cui si vuole accedere
			System.out.println("CR - (Acc. Rem) inviato a SC il nome stanza: "+nomestanza);
			outsc.writeObject(nome); //nome dell'utente che vi vuole accedere
			System.out.println("CR - (Acc. Rem)acc inviato a SC il nome utente: "+nome);
			outsc.writeObject(indirizzo); //indirizzo dell' utente
			System.out.println("CR - (Acc. Rem) inviato a SC indirizzo utente: "+indirizzo);
			outsc.writeObject(new Integer(portaudp)); //porta udp
			System.out.println("CR - (Acc. Rem) inviato a SC porta utente: "+portaudp);
			outsc.writeObject(new Boolean(collegato)); //informazione se l'utente  collegato o meno
			System.out.println("CR - (Acc. Rem) inviato a SC info collegato: "+collegato);
			messaggio=(String)insc.readObject();
			System.out.println("CR - (Acc. Rem) ricevuto da SC risultato: "+messaggio);
			if (messaggio!=null && (messaggio.compareTo(CostantiCom.OK)==0)) {
				porta=((Integer)insc.readObject()).intValue();
				infocopia=new InfoCopia(host,porta);
			}
			if (messaggio!=null && (messaggio.compareTo(CostantiCom.FAIL)==0)){
				System.out.println("CR - (Acc. Rem) Problemi su SC remoto durante l'accesso alla stanza "+nomestanza+" da parte di "+nome);
				infocopia=null;
			}
		}
		catch (Exception e) {
			//per debugging 
			System.out.println("CR - (Acc. Rem) Comunicazione con SC per accesso ad una stanza fallita.");
			infocopia=null;
		}
		//Chiusura del Server di Chat.
		try {
			outsc.close();
			insc.close();
			sc.close();
		}
		catch (IOException ioe) {
			//per debugging 
			System.out.println("CR - (Acc. Rem) Chiusura comunicazione con SC per accesso ad una stanza fallita.");
		}
	return infocopia;
	} //fine accRemoto


} //fine classe

