package scthreads;

import java.util.*;
import java.net.*;
import java.io.*;

import mainpack.*;
import strutturedati.*;
import utilita.*;

/**
Autore:	Terenzio<BR>
Data:		01-09-2000<BR>
Vers:		1.0<BR>
Classe che gestisce gli accessi degli utenti ad certa stanza.
*/

class GestoreStanza extends Thread {
	private MainClass mc;
	private DBStanzeLocali dbs; 
	private String nomestanza;
	private ServerSocket socketdiaccesso;
	private Socket client;
	private int portadiaccesso;
	
	//Costruttore
	
	/**
	Dato il nome della stanza da gestire predispone le variabili private per gestire l'accesso degli
	utenti.
	*/
	public GestoreStanza(MainClass mc, String nomestanza) {
		this.mc=mc;
		dbs=mc.dbstanzelocali;
		this.nomestanza=nomestanza;
		boolean ricercaporta=true;
		// seleziono solo porte sopra la 40000
		int temp=mc.rgen.nextInt(25536)+40000;
		while (ricercaporta) {
			try {
				System.out.println("GS - (Costr) Apertura stanza "+nomestanza);
				socketdiaccesso=new ServerSocket(temp);
				socketdiaccesso.setSoTimeout(CostantiAlt.TIMEOUTSSSTANZA);
				ricercaporta=false;
			}
			catch (SocketException se) {
				//per debugging
				System.out.println("GS - (Costr) Impossibile impostare il timeout della SSocket della stanza "+nomestanza);
			}
			catch (IOException e) {
				//per debugging
				System.out.println("GS - (Cost) Impossibile creare stanza su porta "+temp+" riprovo...");
			}
		}
		portadiaccesso=temp;
	} //fine costruttore

	//Metodi
	
	/**
	Questo metodo riceve le connessioni degli utenti alla stanza, ne verifica i diritti di accesso, 
	attiva se  il caso il meccanismo di recovery per la stanza stessa e genera un GestoriUtenteStanza
	per ogni utente che ha diritto di accesso alla stanza stessa.
	*/
	public void run() {
		System.out.println("GS - (run) partito per stanza: "+nomestanza);
		boolean stanzainuso=true;
		while (stanzainuso) {
			try {
				client=socketdiaccesso.accept();
				GestoreUtenteStanza gus=new GestoreUtenteStanza(mc,nomestanza, client);
				gus.start();
			}
			catch (InterruptedIOException iioe) {
				//verifico se la stanza  ancora viva.
				String[] nomiutenti=dbs.getNomiUtenti(nomestanza);
				if (nomiutenti==null) stanzainuso=false; //la stanza  morta
				continue;
			}
			catch (IOException ioe) {
				//per debugging
				System.out.println("GS - (run) Impossibile fare la accept per stanza "+nomestanza);
				stanzainuso=false;
			}
			
		} //fine while
		System.out.println("GS - (run) "+nomestanza+" sta per morire per mancanza di utenti");
	 } //fine run
	
	/**
	Restituisce la porta della stanza gestita.
	*/
	public int getPorta() {
		return portadiaccesso;
	} //fine getPorta
 
	
} //fine classe

