package scthreads;



import java.util.*;
import java.net.*;
import java.io.*;

import mainpack.*;
import strutturedati.*;
import recovery.*;
import utilita.*;

/**
Autore:	Terenzio<BR>
Data:		01-09-2000<BR>
Vers:		1.0<BR>
Gestisce la comunicazione di un utente con gli altri utenti dentro la stanza.
*/

class GestoreUtenteStanza extends Thread {
	private MainClass mc;
	private DBStanzeLocali dbs;
	private Socket client;
	private DatagramSocket socketdiuscita;
	private ObjectInputStream ois;
	private String nomeutente,nomestanza,messaggio=null;
	private byte[] buffer=null;
	
	//Costruttore
	
	
	public GestoreUtenteStanza(MainClass mc,String nomestanza, Socket client) {
		this.mc=mc;
		dbs=mc.dbstanzelocali;
		this.nomestanza=nomestanza;
		this.client=client;
	} //fine costruttore

	/**
	Invia i messaggi dell' utente gestito agli altri utenti nella stanza. Se l'utente esce dalla stanza
	o dal sistema lo deregistra dalla stanza locale e dalla copia remota.
	*/
	public void run() {
		System.out.println("GUS - (run) partito per stanza:"+nomestanza);
		int controllo=0;
		boolean ok=true;
		Receiver datiutente=null;
		Receiver[] utentistanza=null;
		// Apertura del flusso di input dal Client
		try {
			ois=new ObjectInputStream(client.getInputStream());
		  System.out.println("GUS - (run) Stream di input aperto per utente.");
			nomeutente=(String)ois.readObject();  //leggo il nome dell'utente
			System.out.println("GUS - (run) Letto nome utente: "+nomeutente);
		}
		catch (Exception e) {
			//per debugging
			System.out.println("GUS - (run) Impossibile leggere i dati inviati da C");
			ok=false;
		}
		try {
			socketdiuscita=new DatagramSocket();
			System.out.println("GUS - (run) Aperta DatagramSocket per "+nomeutente+" in stanza "+nomestanza);
		}
		catch (SocketException se) {
			//per debugging
			System.out.println("GUS - (run) Impossibile creare la DatatagramSocket per "+nomeutente+" in "+nomestanza);
			socketdiuscita=null;
			ok=false;
		}	
		//Comunicazione con il Client per la verifica dei diritti di accesso
		if (ok) {
			System.out.println("GUS - (run) Verifica diritti accesso stanza per: "+nomestanza+" "+nomeutente+" "+client.getInetAddress());
			controllo=dbs.verifyUtenteCollegato(nomestanza,nomeutente,client.getInetAddress());
			System.out.println("GUS - (run) Risultato verifica diritti utente: "+controllo);
			switch (controllo) {
				case 0:
					System.out.println("GUS - (run) Utente senza diritti di accesso alla stanza "+nomestanza);
					ok=false;
					break;
				case 1: //ok vale gi true;
					System.out.println("GUS - (run) Utente con diritto di accesso alla stanza "+nomestanza+" come collegato.");
					break;
				case 2:
					System.out.println("GUS - (run) Utente con diritto di accesso alla stanza "+nomestanza+" ma non collegato. E' necessario il recovery della stanza!");
					ok=comunicaInfoRecovery();
					break;
				case 3:
					System.out.println("GUS - (run) Assurdo la stanza a cui l'utente vuole accedere non esiste!");
					ok=false;

			}
		}
		// Comunicazione con il Client - Scambio di messaggi
			while (ok) {
				System.out.println("GUS - (run) Sistema pronto a ricevere messaggi da "+nomeutente+ " in stanza "+nomestanza);
				try {
					//Ricezione
					messaggio=(String)ois.readObject();
					System.out.println("GUS - (run) Letto messaggio utente: "+messaggio);
					messaggio=CostantiCom.TESTO.concat(" ").concat(messaggio);
					buffer=messaggio.getBytes();
				}
				catch (Exception e) {
					//per debugging
					System.out.println("GUS - (run) Problemi nella ricezione del messaggio che "+nomeutente+" in "+nomestanza+" vuole inviare agli altri utenti.");
					System.out.println("GUS - (run) Eliminazione locale utente da stanza");
					// IMPLEMENTARE CANCELLAZIONE REMOTA
					int delremoto=deregistrazioneRemota(nomestanza,nomeutente);
					//Cancellazione utente dal database delle stanze locale
					int delresult=dbs.delUtenteFromStanza(nomestanza,nomeutente);
					System.out.println("GUS - (run) Risultato eliminazione locale utente "+nomeutente+" da stanza "+nomestanza+":"+delresult);
					if  (delremoto==delresult) System.out.println("GUS - Deregistrazione utente avvenuta correttamente.");
					else System.out.println("GUS - (run) PROBLEMI NELLA DEREGITRAZIONE DELL' UTENTE! - INCOERENZA NEI DB!");
					switch (delresult) {
						case 0:
							//per debugging
							System.out.println("GUS - (run) Assurdo stanza "+nomestanza+" non trovata.");
							break;
						case 1:
							//per debugging
							System.out.println("GUS - (run) Assurdo stanza "+nomestanza+" trovata ma utente "+nomeutente+" non trovato.");
							break;
					case 2:
						//per debugging
						System.out.println("GUS - (run) Stanza "+nomestanza+" trovata e utente "+nomeutente+" eliminato.");
						break;
					case 3:
						//per debugging
						System.out.println("GUS - (run) Stanza "+nomestanza+" e "+nomeutente+" eliminati.");
						break;
					}
					ok=false;
					continue;
				}
				if (!ok) continue;
				//Invio
				utentistanza=dbs.getReceivers(nomestanza);
				if (utentistanza==null) {
					System.out.println("GUS - (run) Non ho trovato nessun utente nella stanza "+nomestanza);
					ok=false;//problemi
					continue;
				}
				try {
					for (int i=0; i<utentistanza.length; i++) {
						InetAddress indirizzo=utentistanza[i].getIndirizzo();
						int porta=utentistanza[i].getPorta();
						DatagramPacket pacchetto=new DatagramPacket(buffer,buffer.length,indirizzo,porta);
						socketdiuscita.send(pacchetto);
					}
				}
				catch (IOException ioe) {
					System.out.println("GUS - (run) Impossibile inviare uno dei DatagramPacket che "+nomeutente+" vuole inviare.");
				}
			} //fine while
			chiudiConnessione();
	} //fine run
	
	/**
	Chiude la connessione con il Client al momento della sua uscita dalla stanza o dal sistema. 
	*/
	private void chiudiConnessione() {
		try {
			ois.close();
			client.close();
		}
		catch (IOException ioe2) {
			//per debugging
			System.out.println("GUS - (close) Impossibile chiudere la socket TCP a cui  connesso C.");
		}
		if (socketdiuscita!=null) {
			socketdiuscita.close();
			System.out.println("GUS - (close) Impossibile chiudere la socket UDP usata per l'invio dei messaggi di "+nomeutente);
		}
	} //fine chiudiConnessione
	
	/**
	Metodo che comunica il pacchetto di recovery al client che l'ha scatenato. Ritorna true se il pacchetto  stato
	inviato, false altrimenti.
	*/
	private boolean comunicaInfoRecovery() {
		//inizio sezione critica
		while (!dbs.startRecovery(nomestanza)) {
			System.out.println("GUS - (Inf Rec) Altro recovery in atto per stanza "+nomestanza+" attendo...");
			dbs.setUtenteCollegato(nomestanza,nomeutente,true);
			int dormi=mc.rgen.nextInt(1001)+1000;
			try {
				sleep(dormi);
			}
			catch (InterruptedException ie) {
				System.out.println("GUS - (Inf Rec) La InterruptedExcpetion ha svegliato la sleep");
			}
		}
		System.out.println("GUS - (Inf Rec) Iniziata procedura di comunicazione informazioni di recovery a "+nomeutente);
		RecoveryAlgorithm ra=new RecoveryAlgorithm(nomestanza,dbs,mc.rgen,nomeutente);
		InetAddress indirizzonuovacopia=ra.getServer();
		int porta=ra.getPorta();
		System.out.println("GUS - (Inf Rec) La nuova copia di "+nomestanza+"  su "+indirizzonuovacopia+" alla porta "+porta);
		dbs.finishRecovery(nomestanza);
		//fine sezione critica
		messaggio=CostantiCom.SIST.concat(" i=").concat(indirizzonuovacopia.getHostAddress()).concat("p=").concat(Integer.toString(porta));
		buffer=messaggio.getBytes();
		DatagramPacket dprec=new DatagramPacket(buffer,buffer.length,client.getInetAddress(),dbs.getReceiver(nomestanza,nomeutente).getPorta());
		try {
			socketdiuscita.send(dprec);
			System.out.println("GUS - (Inf Rec) Informazioni sulla nuova copia di "+nomestanza+" inviate a "+nomeutente);
		}
		catch (IOException ioe) {
			//per debugging
			System.out.println("GUS - (run) Errore nell' invio del DatagramPacket di recovery per utente "+nomeutente);
			return false;  //cos esco dal while
		}
		return true;
	} //fine comunicaInfoRecovery
	
	/**
	Dato il nome di un utente e quello di una stanza elimina l'utente dalla stanza remota, eventualmente elimina
	la stanza.Ritorna: <BR>  
	-2 se non ci si riesce ad effetuare correttamente l'operazione con il SC remoto.<BR>
	-1 se le informazioni relative alla copia della stanza non vengono trovate.<BR>
	0  se l'operazione remota fallisce.<BR>
	2	 se sia la stanza che l'utente vengono trovati e l'utente viene eliminato.<BR>
	3  se sia la stanza che l'utente vengono trovati e sia l'utente che la stanza vengono eliminate.<BR>
	*/
	private int deregistrazioneRemota(String nomestanza, String nomeutente) {
		Socket scremoto;
		ObjectInputStream scois;
		ObjectOutputStream scoos;
		String risultato="NESSUN RISULTATO";
		InfoCopia copiaremota=dbs.getInfoCopia(nomestanza);
		if (copiaremota==null) {
			System.out.println("GUS (Drg remota) Informazioni su copia remota non trovate.");
			return -1;
		}
		//connessione a Server di Chat remoto con la copia della stanza
		try {
			scremoto=new Socket(copiaremota.getIndirizzo(),CostantiAlt.SCFORSC);
			scoos=new ObjectOutputStream(scremoto.getOutputStream());
			scois=new ObjectInputStream(scremoto.getInputStream());
		}
		catch (IOException ioe) {
			System.out.println("GUS (Drg remota) Impossibile acquisire l'I/O verso il SC remoto "+copiaremota.getIndirizzo());
			return -2;
		}
		//comunicazione con il Server di Chat remoto per la deregistrazione
		try {
			scoos.writeObject(CostantiCom.USEREXIT);
			scoos.writeObject(nomestanza);
			scoos.writeObject(nomeutente);
			risultato=(String)scois.readObject();
			System.out.println("GUS (Drg remota) Letto risultato op. remota: "+risultato);
		}
		catch (Exception e) {
			System.out.println("GUS (Drg remota) Problemi di comunicazione con il SC remoto "+copiaremota.getIndirizzo());
			return -2;
		}
		//chiusura comunicazione con il Server di Chat remoto
		try {
			scoos.close();
			scois.close();
			scremoto.close();
		}
		catch (IOException ioe) {
			System.out.println("GUS (Drg remota) Problemi chiuysura comunicazione con il SC remoto "+copiaremota.getIndirizzo());
		}
		if (risultato.equals(CostantiCom.USDEL)) return 2;
		if (risultato.equals(CostantiCom.USEROOMDEL)) return 3;
		if (risultato.equals(CostantiCom.FAIL)) return 0;
		return -2;
	}  //fine deregistrazioneRemota
		
		
} //fine classe

