package scthreads;


import java.util.*;
import java.net.*;
import java.io.*;

import mainpack.*;
import strutturedati.*;
import utilita.*;

/**
Autore:	Terenzio<BR>
Data:		01-09-2000<BR>
Vers:		1.0<BR>
Classe che permette di risolvere le richieste degli altri Server di Chat.
*/

public class ServerChatReplier extends Thread {
	MainClass mc;
	DBStanzeLocali dbs;
	Socket client; //porta a cui  connesso il Server di Chat che effettua la richiesta
	ObjectInputStream ois; //da qui leggo le informaziomi inviate dal Server di Chat
	ObjectOutputStream oos;//qui invio le informazioni verso il Server di Chat
	
	//Costruttore
	
	/**
	Apre i flussi di I/O verso il Server di Chat remoto.
	*/
	public ServerChatReplier(MainClass mc, Socket client) {
		this.mc=mc;
		this.client=client;
		dbs=mc.dbstanzelocali;
		// Apro gli stream di I/O verso il Server di Chat che ha effettuato la richiesta
		try {
			ois=new ObjectInputStream(client.getInputStream());
			oos=new ObjectOutputStream(client.getOutputStream());
		}
		catch (IOException ioe1) {
			//per debugging
			System.out.println("SCR - (Costr) Impossibile aprire gli stream di I/O.");
			try {
				ois.close();
				oos.close();
				client.close();
			}
			catch (Exception e) {
				//per debugging
				System.out.println("SCR - (Costr) Impossibile chiudere la socket e seguito di errori di I/O.");
			}
			ois=null;
			oos=null;
			client=null;
		}
	} //fine costruttore
	
	// Metodi
	
	/**
	Verifica il tipo di richiesta effettuata dal Server di Chat remoto e chiama i metodi opportuni.
	*/
	public void run() {
		System.out.println("SCR - (run) partito");
		String info="NON INIZIALIZZATA A NULLA";
		boolean ok=true;
		if (ois!=null && oos!=null && client!=null) { //ha senso procedere
			try {
				info=(String)ois.readObject();
			}
			catch (Exception e) {
				//per debugging
				System.out.println("SCR - (run) Impossibile leggere la richiesta del SC remoto.");
				try {
					ok=false;  //non ha pi senso procedere
					ois.close();
					oos.close();
					client.close();
				}
				catch (IOException ioe) {
					//per debugging
					System.out.println("SCR - (run) E1 Impossibile chiudere la socket e/o i relativi stream.");
				}
			}
			if (ok) {
				System.out.println("SCR - (run) Ricevuto la seguente richiesta: "+info);
				if (info.equals(CostantiCom.ROOMREQS)) richiestaListaStanzeLocali();
				if (info.equals(CostantiCom.ROOMCRS)) accessoCreazioneStanza();
				if (info.equals(CostantiCom.ROOMACCS)) accessoStanza();
				if (info.equals(CostantiCom.USEREXIT)) eliminazioneUtenteDaStanza();
				//terminate le operazioni del caso chiudo gli stream e la socket.
				try {
					ois.close();
					oos.close();
					client.close();
					System.out.println("SCR - (run) Chiusura comunicazione con altro SC avvenuta correttamente.");
				}
				catch (IOException ioe) {
					//per debugging
					System.out.println("SCR - (run) Impossibile chiudere comunicazione con SC.");
				}
			}
		} //fine if
	} //fine run

	/**
	Risolve la richiesta della lista delle stanze locali fornendo al Server di Chat remoto la lista dei
	nomi delle stanze attualmente nel DBStanzeLocali. Ritorna true se l'operazione va a buon fine false
	altrimenti.
	*/
	private boolean richiestaListaStanzeLocali() {
		String[] listanomistanze=dbs.getNomiStanze();
		if (listanomistanze==null) {
			//la lista  vuota
			try { 
				System.out.println("SCR - (List Req.) Invio messaggio LISTAVUOTA.");
				oos.writeObject(new Integer(CostantiCom.LISTAVUOTA));
				return true;
			}
			catch (Exception e) {
				//per debugging
				System.out.println("SCR - (List Req.) Impossibile inviare messaggio LISTAVUOTA.");
				return false;
			}
		}
		// la lista non  vuota
			try { 
				oos.writeObject(new Integer(listanomistanze.length));
				for(int i=0; i<listanomistanze.length; i++) oos.writeObject(listanomistanze[i]);
				return true;
			}
			catch (IOException e) {
				//per debugging
				System.out.println("SCR - (List Req.) Impossibile inviare la lista delle stanze locali.");
				return false;
			}
	} //fine richiestaListaStanzeLocali
	
	/**
	Risolve la richiesta di creazione di una stanza verificando se la stanza esiste e creando il 
 	gestore della stanza stessa se  il caso, invoca poi il metodo necessario per far accedere un utente
	alla stanza stessa. Ritorna true se l'operazione va a buon fine false	altrimenti.
	*/
	private boolean accessoCreazioneStanza() {
		String nomestanza=null;
		int portacopia,portaudp,portastanza=-1;
		boolean collegato=false,esiste;
		try {
			nomestanza=(String)ois.readObject(); //nome della stanza da creare
			System.out.println("SCR - (AC Stanza) Ricevuto nomestanza da creare: "+nomestanza);
			portacopia=((Integer)ois.readObject()).intValue(); //porta della copia
			System.out.println("SCR - (AC Stanza) Ricevuta porta copia stanza sul SC remoto: "+portacopia);
		}
		catch (Exception e) {
			//per debugging
			System.out.println("SCR - (AC Stanza) Impossibile leggere i dati di creazione di una stanza.");
			return false;
		}
		esiste=dbs.verifyStanza(nomestanza);
		if (!esiste) { //la stanza non esiste
			GestoreStanza gestorestanza=new GestoreStanza(mc, nomestanza);
			portastanza=gestorestanza.getPorta();
			dbs.addStanza(new InfoStanza(nomestanza,portastanza,client.getInetAddress(),portacopia));
			gestorestanza.start(); //avvio la stanza
		}
		return accessoStanza(nomestanza);
	}
			
	/**
	Risolve la richiesta di accesso di un utente in una stanza verificando se la stanza esiste. Ritorna
 	true se l'operazione va a buon fine false	altrimenti.
	*/
	private boolean accessoStanza() {
		String nomestanza=null;
		try {
			nomestanza=(String)ois.readObject(); //nome della stanza da creare
			System.out.println("SCR - (A Stanza 1) Letto nome stanza a cui accedere: "+nomestanza);
		}
		catch (Exception e) {
			//per debugging
			System.out.println("SCR - (A Stanza 1) Impossibile leggere il nome della stanza da accedere.");
			return false;
		}
		return accessoStanza(nomestanza);
	}	//fine accessoStanza
	
	/**
	Inserisce un utente nella stanza in cui vuole entrare. Ritorna true se l'operazione va a buon fine
	false altrimenti.
	*/
	private boolean accessoStanza(String nomestanza) {
		String nomeutente=null;
		InetAddress indirizzo=null;
		int portaudp=-1;
		boolean collegato=false,ok;
		
		try {
			nomeutente=(String)ois.readObject(); //il nome dell' utente che vuole accedere
			System.out.println("SCR - (A Stanza 2) Letto nome utente che vuole accedere: "+nomeutente);
			indirizzo=(InetAddress)ois.readObject(); //il suo indirizzo ip
			System.out.println("SCR - (A Stanza 2) Letto indirizzo utente che vuole accedere: "+indirizzo);
			portaudp=((Integer)ois.readObject()).intValue(); //la porta su cui ascolter i messsaggi
			System.out.println("SCR - (A Stanza 2) Letto porta udp utente che vuole accedere: "+portaudp);
			collegato=((Boolean)ois.readObject()).booleanValue(); //informazione se  collegato
			System.out.println("SCR - (A Stanza 2) Letto informazioni su collegato: "+collegato);

		}
		catch (Exception e) {
			//per debugging
			System.out.println("SCR - (A Stanza 2) Impossibile leggere il nome della stanza da accedere.");
			return false;
		}
		
		ok=dbs.addUtenteToStanza(nomestanza,new InfoUtente(nomeutente,indirizzo,portaudp,collegato));
		System.out.println("SCR -  (A Stanza 2) Risultato aggiunta utente:"+ok);
		if (ok) try {
							oos.writeObject(CostantiCom.OK);
							System.out.println("SCR -  (A Stanza 2) Invio a CR remoto l'OK di accesso avvenuto"); 
							oos.writeObject(new Integer(dbs.getPortaStanza(nomestanza)));
							System.out.println("SCR - (A Stanza 2) Invio a CR remoto la porta della stanza "+nomestanza+" che  "+dbs.getPortaStanza(nomestanza)); 
							return true; //accesso riuscito
						}
						catch (IOException ioe) {
							//per debugging
							System.out.println("SCR - (A Stanza 2) Impossibile inviare i dati relativi all accesso dell utente nella stanza locale.");
							return false;
						}
		// se arrivo qui allora ok valeva false
		try {
			oos.writeObject(CostantiCom.FAIL);
		}
		catch (IOException ioe) {
			//per debugging
			System.out.println("SCR - (A Stanza 2) Impossibile inviare il CostantiCom.FAIL");
		}
		//qualche cosa  andato storto
		return false;	
	} //fine accessoStanza
		
	/**
	Risolve la richiesta di eliminazione di un dato utente da una data stanza. Se l'utente  l'ultimo
	utente nella stanza provvede ad eliminare la stanza stessa dal DBStanzeLocali. Ritorna true se 
	l'operazione va a buon fine, false altrimenti.
	*/
	private boolean eliminazioneUtenteDaStanza() {
		String nomestanza=null,nomeutente=null;
		int retvalue=-1;
		// lettura dati
		try {
			nomestanza=(String)ois.readObject();
			System.out.println("SCR - (Drg Ut.) Letto nome stanza: "+nomestanza);
			nomeutente=(String)ois.readObject();
			System.out.println("SCR - (Drg Ut.) Letto nome utente: "+nomeutente);
		}
		catch (Exception e) {
			//per debugging
			System.out.println("SCR - (Drg. Ut.) Impossibile leggere dati per richiesta di eliminazione di un utente da una stanza.");	
			try {
				oos.writeObject(CostantiCom.FAIL);
			}
			catch (IOException ioe2) {
				System.out.println("SCR - (Drg Ut.) Impossibile inviare il FAIL!");
			}
			return false;
		}
		retvalue=dbs.delUtenteFromStanza(nomestanza,nomeutente);
		try {
			if (retvalue==0 || retvalue==1) {
				oos.writeObject(CostantiCom.FAIL);
				System.out.println("SCR - (Drg Ut.) Operazione sul DB locale errata, inviato FAIL.");
				return false;
			}
			if (retvalue==2) {
				oos.writeObject(CostantiCom.USDEL);
				System.out.println("SCR - (Drg Ut.) Eliminato "+nomeutente+" dal DB locale, inviato USDEL.");
			}
			if (retvalue==3) {
				oos.writeObject(CostantiCom.USEROOMDEL);
				System.out.println("SCR - (Drg Ut.) Eliminato "+nomeutente+" e stanza "+nomestanza+" dal DB locale, inviato USEROOMDEL.");
			}
			return true;
		}
		catch (IOException ioe) {
			//per debugging
			System.out.println("SCR - (Drg Ut.) Impossibile inviare costante di risoposta.");
			return false;		
		}	
	} //fine eliminazioneUtenteDaStanza


} //fine classe

