package strutturedati;

import java.util.*;
import java.net.*;

/**
Autore:	Terenzio<BR>
Data:		01-09-2000<BR>
Vers:		1.0<BR>
Realizza la cache delle stanze presenti nel sistema.
*/

 public class CacheStanze {
	private Date timestamp;
	private ArrayList listastanze;
	private boolean aggiornamentoincorso;

	//Costruttore
	
	/**
	Inizializza una nuova cache delle stanze.
	*/
	public CacheStanze() {
		timestamp=new Date();
		listastanze=new ArrayList();
		aggiornamentoincorso=false;
	} //fine costruttore
	
	//Metodi
	
	/**
	Dato l'indice di posizione ritorna la stanza in quella posizione nella lista, ritorna null se tale
	posizione non esiste.
	*/
	private InfoGlobaleStanza getStanza(int i) {
		int dim=listastanze.size();
		if (i<dim) return (InfoGlobaleStanza)listastanze.get(i);
		return null;
	} //fine getStanza
	
	/**
	Dato il nome di una stanza ne recupera le informazioni dalla cache stanze.Ritorna null se la stanza 
	non viene trovata.
	*/
	public InfoGlobaleStanza getStanza(String nome) {
		InfoGlobaleStanza elem;
		String nomeelem;
		int dim,temp;
		dim=listastanze.size();
		for (int i=0; i<dim; i++) {
			elem=(InfoGlobaleStanza)listastanze.get(i);
			nomeelem=elem.getNome();
			temp=nomeelem.compareToIgnoreCase(nome);
			if (temp==0) return elem;
			if (temp>0) return null;
		}
		return null;
	} //fine getStanza
		
	/** 
	Aggiunge le informazioni di base relative ad una stanza nel sistema in ordine alfabetico secondo
 	il suo nome. Attenzione questo metodo non va utilizzato sulla vera cache stanze perch potenzialmente
	pericoloso. Utilizzarlo solo su proprie istanze private. 
	*/
	public boolean addStanza(String nomestanza, InetAddress server) {
		InfoGlobaleStanza stanza,elem;
		String nomeelem; 
		int dim,i,temp;
		elem=getStanza(nomestanza);
		if (elem!=null) return elem.addServer(server);
		dim=listastanze.size();
		for (i=0; i<dim; i++) {
			elem=getStanza(i);
			nomeelem=elem.getNome();
			temp=nomeelem.compareToIgnoreCase(nomestanza);
			if (temp==0) return false;
			if (temp>0) break;
		}
		stanza=new InfoGlobaleStanza(nomestanza, server);
		listastanze.add(i,stanza);
		return true;		
	} //fine addStanza
	
	/**
	Fornisce i nomi di tutte le stanze presenti nella cache in ordine alfabetico.
 	Ritorn null in caso di fallimento
	Utile per implementare l'operazione di richiesta lista stanze.
	*/
	public synchronized String[] getNomiStanze() {
		String result[]=null;
		int dim;
		dim=listastanze.size();
		if (dim==0) return null;
		else result=new String[dim];
		for (int i=0; i<dim; i++) result[i]=getStanza(i).getNome();
		return result;
	} //fine getNomiStanze
	
	/** 
	Verifica se e necessario un aggiornamento.Prima viene verificato che nessun altro thread stia
  gi aggiornando la Cache e poi viene verificato se la cache  pi vecchia di delay millisecondi.
	Se entrambe le condizioni risultano verificate viene predisposto l'aggiornamento ponendo a true 
	aggiornamento in corso:  a carico del chiamante effetturare l'aggiornamento.
	Ritorna true se  necessario fare l'aggiornamento false altrimenti.
	*/
	public synchronized boolean verifyAggiornamento(long delay) {
		Date adesso=new Date();
		if ( (adesso.getTime()-timestamp.getTime()>delay) && !aggiornamentoincorso ) return aggiornamentoincorso=true;
		return false;
	} //fine verifyAggiornamento
	
	/**
	Sostituisce la vecchia cache con una nuova pi aggiornata.
	Questo metodo va utilizzato in sostituzione del normale assegnamento sulla CacheStanze condivisa.
	*/
	public synchronized void setAggiornamento(CacheStanze newcache) {
		listastanze=newcache.listastanze;
		timestamp=new Date();
		aggiornamentoincorso=false;
	} //fine setAggiornamento
	
	
}	//fine classe

