package strutturedati;

import java.util.*;
import java.net.*;

/**
Autore:	Terenzio<BR>
Data:		01-09-2000<BR>
Vers:		1.0<BR>
Realizza il DB delle Stanze Locali. Poich tale classe viene acceduta da pi thread il suo accesso
viene controllato attraverso i metodi synchronized. Solo questi metodi possono essere invocati sulla
istanza globale della classe presente in mainpack.MainClass. I metodi public ma non synchronized 
possono essere invocati solo su istanze private di tale oggetto.
*/

public class DBStanzeLocali {
	ArrayList dbstanzelocali;
	
	//Costruttore
	
	/**
	Crea un DataBase delle stanze locali inizialmente vuoto.
	*/
	public DBStanzeLocali() {
		dbstanzelocali=new ArrayList();
	} //fine cotruttore
	
	//Metodi
		
	/**	
	Dato il nome di una stanza la cerca e nella lista. Restituisce al chiamante null nel caso la stanza
	non venga trovata. Metodo protetto perch utilizzato solo internamente alla classe.
	*/
	protected InfoStanza getStanza(String nome) {
		String nomestanza;
		InfoStanza elem=null;
		int dim,temp;
		dim=dbstanzelocali.size();
		for (int i=0; i<dim; i++) {
			elem=(InfoStanza)dbstanzelocali.get(i);
			temp=nome.compareToIgnoreCase(elem.getNome());
			if (temp>0) return null;
			if (temp==0) return elem;
		}
		return null;
	} //fine getStanza
	
	/**
	Dato l'indice di posizione ritorna la stanza in quella posizione nella lista, ritorna null se tale
	posizione non esiste.
	*/
	protected InfoStanza getStanza(int i) {
		int dim=dbstanzelocali.size();
		if (i<dim) return (InfoStanza)dbstanzelocali.get(i);
		return null;
	} //fine getStanza
	
	/** 
	Data una Stanza la inserisce nella lista delle stanze locali con ordinamento alfabetico	secondo i 
	nomi delle stanze. Ritorna true se la stanza viene inserita false se  gi esistente per cui non pu
	essere inserita.
	Utile per implementare l'operazione di creazione di una nuova stanza nel sistema.
	*/
	public synchronized boolean addStanza(InfoStanza stanza) {
		String nome,elem;
		int dim,temp;
		nome=stanza.getNome();
		dim=dbstanzelocali.size();
		for (int i=0; i<dim; i++) {
			elem=((InfoStanza)dbstanzelocali.get(i)).getNome();
			temp=nome.compareToIgnoreCase(elem);
			if (temp==0) return false;
			if (temp>0) {
				//aggiungo la stanza nel punto giusto
				dbstanzelocali.add(i,stanza);
				return true;
			}
		}
		//aggiungo la stanza in fondo alla lista
		dbstanzelocali.add(stanza);
		return true;
	} //fine addStanza	

	/** 
	Dato in nome di una stanza e le informazioni relative ad un utente aggiunge l'utente alla stanza. 
	Ritorna true se l'utente viene effettivamente aggiunto alla stanza false altrimenti. 
	(false significa che la stanza non  stata trovata nel database o che l'utente era gi nella stanza)  
	Utile per inplementare l'operazione di ingresso di un utente in una stanza.
	*/
	public synchronized boolean addUtenteToStanza(String nomestanza, InfoUtente utente) {
		InfoStanza stanza;
		stanza=getStanza(nomestanza);
		if (stanza!=null) {
			System.out.println("DBS Stanza "+nomestanza+" trovata");
			return stanza.addUtente(utente);
		}
		return false; //problemi
	} //fine addUtenteToStanza
	
	/** 
	Verifica se l'utente  collegato al Server di Chat o al Server che contiene la copia della stanza in
	cui l'utente vuole entrare.
	Ritorna:<BR>
	0 Se l'utente non ha diritto di accesso alla stanza.<BR>
	1 Se l'utente ha diritto di accesso come utente collegato.<BR>
	2 Se l'utente ha diritto di accesso ma dovrebbe essere collegato alla copia. In questo caso bisogna
		avviare il recovery.<BR>
	3 Se la stanza non esiste.<BR>
	*/
	public synchronized int verifyUtenteCollegato(String nomestanza, String nomeutente, InetAddress indirizzoutente) {
		InfoStanza stanza=getStanza(nomestanza);
		if (stanza==null)	return 3;
		return stanza.verifyUtenteCollegato(nomeutente,indirizzoutente);
	} //fine verifyUtenteCollegato
	
	/**
	Dato il nome di un Utente ne setta il campo collegato.
	Ritorna true se l'operazione  riuscita, false altrimenti.
	*/	
	public synchronized boolean setUtenteCollegato(String nomestanza, String nomeutente, boolean collegato) {
		InfoStanza stanza=getStanza(nomestanza);
		if (stanza==null) return false; //la stanza non esiste;
		return stanza.setUtenteCollegato(nomeutente,collegato);
	} //fine setUtenteCollegato
	
	/**
	Metodo che permette l'esecuzione di un recovery per una stanza. Se recovery vale false allora il
	recovery  consentito e tale campo viene settato a true in modo tale da impedire ad altri thread
	di avviare altri recovery. Questo metodo evita il rischio che a fronte della caduta della copia 
	della stanza ne venga creata pi di una copia. Tale metodo ritorna true se il recovery pu essere
	avviato false altrimenti.
	*/
	public synchronized boolean startRecovery(String nomestanza) {
		InfoStanza stanza=getStanza(nomestanza);
		if (stanza==null) return false; //stanza non trovata!
		return stanza.startRecovery();
	} //fine startRecovery
	
	/**
	Metodo che permette di confermare il termine della procedura di recovery per una data stanza.
	Va invocato al termine della procedura stessa e ritorna il nuovo valore del campo recovery per 
	tale stanza.
	*/
	public synchronized boolean finishRecovery(String nomestanza) {
		InfoStanza stanza=getStanza(nomestanza);
		if (stanza==null) return false; //stanza non trovata!
		return stanza.finishRecovery();
	} //fine finishRecovery
	
	/**
	Setta l'indirizzo e la porta TCP della copia della stanza nel sistema.
	Metodo utile per impostare tali valori dopo un recovery.
	Ritorna true se l'operazione va a buon fine false altrimenti.
	*/
	public synchronized boolean setInfoCopiaStanza(String nomestanza,InfoCopia infocopia) {
		InetAddress indirizzo=infocopia.getIndirizzo();
		int porta=infocopia.getPorta();
		InfoStanza stanza=getStanza(nomestanza);
		if (stanza==null) return false;
		stanza.setInfoCopia(indirizzo,porta);
		return true;
	} //fine setInfoCopiaStanza
	
	/**
	Setta l'indirizzo e la porta TCP della copia della stanza nel sistema.
	Metodo utile per impostare tali valori dopo un recovery.
	Ritorna true se l'operazione va a buon fine false altrimenti.
	*/
	public synchronized boolean setInfoCopiaStanza(String nomestanza, InetAddress indirizzo, int porta) {
		InfoStanza stanza=getStanza(nomestanza);
		if (stanza==null) return false;
		stanza.setInfoCopia(indirizzo,porta);
		return true;
	} //fine setInfoCopiaStanza
	
	/**
	Dato il nome di una stanza e di un utente ritorna l'indirizzo e la porta UDP di ascolto del client
	dell'utente. Il metodo ritorna null se la stanza o l'utente non esistono.
	Questo metodo  utile per inviare il messaggio di recovery all'utente che l'ha scatenato.
	*/
	public synchronized Receiver getReceiver(String nomestanza, String nomeutente) {
		InfoStanza stanza=getStanza(nomestanza);
		if (stanza==null) return null;
		return stanza.getReceiver(nomeutente);
	} //fine getReceiver
	
	/**
	Ritorna l'indirizzo e la porta UDP di tutti i client presenti nella stanza con tale nome. 
	Il metodo ritorna null se la stanza non esiste o  vuota.
	Questo metodo  utile per rintracciare gli utenti a cui inviare un messaggio.
	*/
	public synchronized Receiver[] getReceivers(String nomestanza) {
		InfoStanza stanza=getStanza(nomestanza);
		if (stanza==null) return null;
		return stanza.getReceivers();
	} //fine getReceivers
	
	
	/**
	Restituisce la lista dei nomi degli utenti in una data stanza. Ritorna null se tale lista  vuova (assurdo) o se la
	stanza di cui si volgiono i nomi degli utenti non viene trovata.
	*/
	public synchronized String[] getNomiUtenti(String nomestanza) {
		InfoStanza stanza=getStanza(nomestanza);
		if (stanza==null) return null;  //stanza non trovata
		return stanza.getNomiUtenti();
	} //fine getNomiUtenti
	
	/**
	Dato il nome di una stanza restituisce le informazioni relative all' altra sua copia presente nel
	sistema. Ritorna null se la stanza non viene trova all' interno del DB.
	*/
	public synchronized InfoCopia getInfoCopia(String nomestanza) {
		InfoStanza stanza=getStanza(nomestanza);
		if (stanza==null) return null; //stanza non trovata
		return stanza.getInfoCopia();
	} // fine getInfoCopia
	
	/**
 	Restituisce i nomi di tutte le stanze presenti sul server locale. Ritorna null se non vi sono stanze.
	*/
	public synchronized String[] getNomiStanze() {
		String[] nomistanze=null;
		int dim=dbstanzelocali.size();
		if (dim==0) return null;
		nomistanze=new String[dim];
		for (int i=0; i<dim; i++) nomistanze[i]=getStanza(i).getNome();
		return nomistanze;
	}	// fine getNomiStanze
	
	/**
	Dato il nome di una stanza ritorna la porta TCP della stanza sul server locale. Ritorna -1 se la 
	stanza non esiste.
	*/
	public synchronized int getPortaStanza(String nomestanza) {
		InfoStanza infostanza=getStanza(nomestanza);
		if (infostanza==null) return -1;
		return infostanza.getPorta();
	} //fine getPortaStanza
	
	/**
	Dato il nome di una stanza ne verifica la presenza nel database. Ritorna true se la stanza esiste
	false altrimenti.
	*/
	public synchronized boolean verifyStanza(String nomestanza) {
		InfoStanza infostanza=getStanza(nomestanza);
		if (infostanza==null) return false;
		return true;
	} //fine verifyStanza
	
	/**
	Dato il nome di un utente e quello di una stanza elimina l'utente dalla stanza, eventualmente elimina
	la stanza. Ritorna: <BR>  
	0 se la stanza non viene trovata.<BR>
	1 se la stanza viene trovata ma l'utente no.<BR>
	2	se sia la stanza che l'utente vengono trovati e l'utente viene eliminato.<BR>
	3 se sia la stanza che l'utente vengono trovati e sia l'utente che la stanza vengono eliminate.<BR>
 	*/
	public synchronized int delUtenteFromStanza(String nomestanza,String nomeutente) {
		InfoStanza infostanza=getStanza(nomestanza);
		boolean bu,bs=false;
		if (infostanza==null) return 0;
		bu=infostanza.delUtente(nomeutente);
		if (!bu) return 1; //utente non trovato
		if (infostanza.getNumeroUtenti()==0) bs=delStanza(nomestanza);
		if (!bs) return 2;  
		return 3;
	} //fine delUtenteFromStanza
	
	/**
	Dato il nome di una stanza la elimina dal DB. Ritorna true se l'operazione va a buon fine false se
	la stanza non viene trovata.
	*/
	protected boolean delStanza(String nomestanza) {
		InfoStanza stanza=null;
		int dim,temp;
		dim=dbstanzelocali.size();
		temp=0;
		for (int i=0; i<dim; i++) {
			stanza=(InfoStanza)dbstanzelocali.get(i);
			temp=nomestanza.compareToIgnoreCase(stanza.getNome());
				if (temp==0) {
					dbstanzelocali.remove(i);
					return true;
				}
				if (temp>0) return false;
		}
		return false;
	} //fine delStanza
		
	
} //fine classe

