package strutturedati;


import java.net.*;

/**
Autore:	Terenzio<BR>
Data:		01-09-2000<BR>
Vers:		1.0<BR>
Classe che contenere le informazioni relative ad una stanza contenuta sul Server di Chat.
*/

public class InfoStanza {
	private String nome;
	private int porta;
	private InetAddress indirizzocopia;
	private int portacopia;
	private UtentiStanza utenti;
	private boolean recovery;

	//Costruttori
	
	/**
	Costruttore che istanzia una nuova stanza, inizialmente senza utenti ma con gi le informazioni 
	sulla posizione della copia all'interno del sistema impostate correttamente.
	*/
	public InfoStanza(String nome, int porta, InetAddress indirizzocopia, int portacopia) {
		this.nome=nome;
		this.porta=porta;
		this.indirizzocopia=indirizzocopia;
		this.portacopia=portacopia;
		utenti=new UtentiStanza();
		recovery=false;
	}  //fine costruttore
	
	/**
	Costruttore che istanzia una nuova stanza, inizialmente senza utenti senza le informazioni 
	sulla posizione della copia all'interno del sistema impostate. Prima dell' utilizzo della stanza
	 necessario impostare tali informazioni.
	*/
	public InfoStanza(String nome, int porta) {
		this.nome=nome;
		this.porta=porta;
		this.indirizzocopia=null;
		this.portacopia=-1;
		utenti=new UtentiStanza();
		recovery=false;
	} //fine costruttore

	//Metodi
	
	/**
	Ritorna il nome della stanza.
	*/
	protected String getNome() {
		return nome;
	} //fine getNome
	
	/**
	Ritorna la porta della stanza sul server locale.
	*/
	protected int getPorta() {
		return porta;
	} //fine getPorta
	
	/**
	Ritrona le informazioni relative al server remoto che contiene la copia della stanza.
 	*/
	protected InfoCopia getInfoCopia() {
		return new InfoCopia(indirizzocopia, portacopia);
	} //fine getInfoCopia
	
	/**
	Dato il nome di un utente ne ritorna l'indirizzo e la porta UDP. Il metodo ritorna null se l'utente
	non  nella lista.
	Questo metodo  utile per inviare il messaggio di recovery all'utente che l'ha scatenato.
	*/
	protected Receiver getReceiver(String nomeutente) {
		return utenti.getReceiver(nomeutente);
	} //fine getReceiver
	
	/**
	Ritorna l'indirizzo e la porta UDP di tutti i client presenti nella lista. Il metodo ritorna null se
	la lista degli utenti  vuota.
	Questo metodo  utile per rintracciare gli utenti a cui inviare un messaggio.
	*/
	protected Receiver[] getReceivers() {
		return utenti.getReceivers();
	} //fine getReceivers
		
	/**
	Setta l'indirizzo e la porta TCP della copia della stanza nel sistema.
	Metodo utile per impostare tali valori dopo un recovery.
	*/
	protected void setInfoCopia(InetAddress copia, int portacopia) {
		indirizzocopia=copia;
		this.portacopia=portacopia;
	} //fine setInfoCopia
	
	/** 
	Dato un utente lo inserisce nella lista degli utenti ordinata in base al loro nome. Ritorna true se
	l'utente viene effettivamente inserito, false se era gi presente nella stanza.
	*/
	protected boolean addUtente(InfoUtente utente) {
		return utenti.addUtente(utente);
	} //fine addUtente
	
	/**
	Dato il nome di un Utente ne setta il campo collegato.
	Ritorna true se l'operazione  riuscita false se l'utente non esiste.
	*/	
	protected boolean setUtenteCollegato(String nomeutente, boolean collegato) {
		return utenti.setUtenteCollegato(nomeutente,collegato);
	} //fine setUtenteCollegato
	
	/** 
	Verifica se l'utente  collegato al Server di Chat o al Server che contiene la copia della stanza in
	cui l'utente vuole entrare.
	Ritorna:<BR>
	0 Se l'utente non ha diritto di accesso alla stanza.<BR>
	1 Se l'utente ha diritto di accesso come utente collegato.<BR>
	2 Se l'utente ha diritto di accesso ma dovrebbe essere collegato alla copia. In questo caso bisogna
	far scattare il recovery per l'utente in questione.<BR>
	*/
	protected int verifyUtenteCollegato(String nomeutente, InetAddress indirizzoutente) {
		return utenti.verifyUtenteCollegato(nomeutente,indirizzoutente);
	} //fine verifyUtenteCollegato
	
	/**
	Metodo che permette l'esecuzione di un recovery per una stanza. Se recovery vale false allora il
	recovery  consentito e tale campo viene settato a true in modo tale da impedire ad altri thread
	di avviare altri recovery. Questo metodo evita il rischio che a fronte della caduta della copia 
	della stanza ne venga creata pi di una copia. Tale metodo ritorna true se il recovery pu essere
	avviato false altrimenti.
	*/
	protected boolean startRecovery() {
		if (!recovery) return recovery=true;
		return false;
	} //fine startRecovery
	
	/**
	Metodo che permette di confermare il termine della procedura di recovery. Va invocato al termine 
	della procedura stessa e ritorna il nuovo valore del campo recovery.
	*/
	protected boolean finishRecovery() {
		if (recovery) return recovery=false;
		return true; //problemi!
	} //fine finishRecovery
	
	/**
	Restituisce la lista dei nomi degli utenti. Ritorna null se tale lista  vuota.
	*/
	protected String[] getNomiUtenti() {
		return utenti.getNomiUtenti();
	} //fine getNomiUtenti
	
	/**
	Dato il nome di un utente lo elimina dalla stanza. Ritorna true se l'utente viene trovato ed eliminato false
 	altrimenti
	*/
	protected boolean delUtente(String nomeutente) {
		return utenti.delUtente(nomeutente);
	}  //fine delUtente
	
	/**
	Restituisce il numero di utenti nella stanza. Tale numero  0 se non vi sono utenti
	*/
	protected int getNumeroUtenti() {
		return utenti.getListLength();
	} //fine getNumeroUtenti
		
	
} //fine classe
