package strutturedati;

import java.util.*;
import java.net.*;

/**
Autore:	Terenzio<BR>
Data:		01-09-2000<BR>
Vers:		1.0<BR>
Classe che contenere le informazioni relative agli utenti dentro una stanza contenuta sul Server di 
Chat.
*/

public class UtentiStanza {
	private ArrayList utentistanza;
	
	//Costruttore
	
	public UtentiStanza() {
		utentistanza=new ArrayList();
	} //fine cotruttore
	
	//Metodi
	
	/** 
	Dato il nome di un utente ne rintraccia le informazioni negli utenti della stanza. Il metodo ritorna
	null se l'utente non viene trovato.
	*/
	protected InfoUtente getUtente(String nome) {
		InfoUtente utente=null;
		int dim,temp;
		dim=utentistanza.size();
		temp=0;
		for (int i=0; i<dim; i++) {
			utente=(InfoUtente)utentistanza.get(i);
			temp=nome.compareToIgnoreCase(utente.getNome());
			if (temp>0) return null;
			if (temp==0) return utente;
		}
		return null;
	} //fine getUtente
	
	/** 
	Data la posizione delle informazioni di un utente nella lista degli utenti estrae tali informazioni
	dalla lista. Il metodo ritorna null se la posizione non esiste nella lista.
	*/
	protected InfoUtente getUtente(int i) {
		int dim=utentistanza.size();
		if (i<dim) return (InfoUtente)utentistanza.get(i);
		return null;
	} //fine getUtente
	
	/** 
	Il metodo ritorna il numero di utenti nella lista.
	*/
	protected int getListLength() {
		return utentistanza.size();
	} //fine getListLength
	
	/**
	Dato il nome di un utente ne ritorna l'indirizzo e la porta UDP. Il metodo ritorna null se l'utente
	non  nella lista.
	Questo metodo  utile per inviare il messaggio di recovery all'utente che l'ha scatenato.
	*/
	protected Receiver getReceiver(String nomeutente) {
		InfoUtente utente=getUtente(nomeutente);
		if (utente!=null) return utente.getReceiver();
		return null;
	} //fine getReceiver
		
	/**
	Ritorna l'indirizzo e la porta UDP di tutti i client presenti nella lista. Il metodo ritorna null se
	la lista degli utenti  vuota.
	Questo metodo  utile per rintracciare gli utenti a cui inviare un messaggio.
	*/
	protected Receiver[] getReceivers() {
		int dim;
		dim=utentistanza.size();
		if (dim==0) return null;
		Receiver[] receiver=new Receiver[dim];
		for (int i=0; i<dim; i++) receiver[i]=new Receiver(getUtente(i).getIndirizzo(),getUtente(i).getPorta());
		return receiver;
	} //fine getReceivers

	/** 
	Dato un utente lo inserisce nella lista degli utenti ordinata in base al loro nome. Ritorna true se
	l'utente viene effettivamente inserito, false se era gi presente nella stanza.
	*/
	protected boolean addUtente(InfoUtente utente) {
		InfoUtente elem=null;
		String nomeutente=utente.getNome();
		System.out.println("US - Tento di aggiungere l'utente:"+nomeutente);
		int dim,i,temp;
		dim=utentistanza.size();
		System.out.println("US - Nella stanza ci sono gi "+dim+" utenti.");
		temp=0;
		for (i=0; i<dim; i++) {
			elem=(InfoUtente)utentistanza.get(i);
			temp=nomeutente.compareToIgnoreCase(elem.getNome());
			if (temp==0) return false;
			if (temp>0) break;
		}
		System.out.println("US - Aggiungo l'utente:"+nomeutente);
		utentistanza.add(i,utente);
		return true;
	} //fine addUtente
		
	/**
	Dato il nome di un Utente lo setta come collegato al Server.
	Ritorna true se l'operazione  riuscita false se l'utente non esiste.
	*/
	protected boolean setUtenteCollegato(String nome, boolean c) {
		InfoUtente utente=getUtente(nome);
		if (utente==null) return false;
		utente.setCollegato(c);
		return true;
	} //fine setUtenteCollegato
	
	/** 
	Verifica se l'utente  collegato al Server di Chat o al Server che contiene la copia della stanza in
	cui l'utente vuole entrare.
	Ritorna:<BR>
	0 Se l'utente non ha diritto di accesso alla stanza.<BR>
	1 Se l'utente ha diritto di accesso come utente collegato.<BR>
	2 Se l'utente ha diritto di accesso ma dovrebbe essere collegato alla copia. In questo caso bisogna
	far scattare il recovery per l'utente in questione.<BR>
	*/
	protected int verifyUtenteCollegato(String nomeutente, InetAddress indirizzoutente) {
		InfoUtente utente=getUtente(nomeutente);
		
		if (utente==null) return 0;
		if (!utente.verifyIndirizzo(indirizzoutente)) return 0;
		if (utente.verifyCollegato()) return 1;
		return 2;
	} //fine verifyUtenteCollegato
	
	/**
	Restituisce la lista dei nomi degli utenti. Ritorna null se tale lista  vuota.
	*/
	protected String[] getNomiUtenti() {
		String nomiutenti[]=null;
		int dim;
		dim=utentistanza.size();
		if (dim==0) return null;
		nomiutenti=new String[dim];
		for (int i=0; i<dim; i++) {
			nomiutenti[i]=getUtente(i).getNome();
		}
		return nomiutenti;
	} //fine getNomiUtenti
	
	/**
	Dato il nome di un utente lo elimina dalla stanza. Ritorna true se l'utente viene trovato ed eliminato false
 	altrimenti.
	*/
	protected boolean delUtente(String nomeutente) {
		InfoUtente utente=null;
		int dim,temp;
		dim=utentistanza.size();
		temp=0;
		for (int i=0; i<dim; i++) {
			utente=(InfoUtente)utentistanza.get(i);
			temp=nomeutente.compareToIgnoreCase(utente.getNome());
				if (temp==0) {
					utentistanza.remove(i);
					return true;
				}
				if (temp>0) return false;
		}
		return false;
	} //fine delUtente
	

} //fine classe

